@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2011-2013 Dirk-Jan C. Binnema <djcb@djcbsoftware.nl>
+** Copyright (C) 2011-2020 Dirk-Jan C. Binnema <djcb@djcbsoftware.nl>
 **
 ** This program is free software; you can redistribute it and/or modify it
 ** under the terms of the GNU General Public License as published by the
@@ -17,19 +17,20 @@
 **
 */
 
-#if HAVE_CONFIG_H
 #include <config.h>
-#endif /*HAVE_CONFIG_H*/
-
 #include <locale.h>
 
 #include <glib-object.h>
+
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wredundant-decls"
 #include <libguile.h>
+#pragma GCC diagnostic pop
 
 #include <mu-runtime.h>
 #include <mu-query.h>
 #include <mu-runtime.h>
-#include <mu-store.h>
+#include <mu-store.hh>
 #include <mu-query.h>
 #include <mu-msg.h>
 
@@ -89,7 +90,7 @@ mu_guile_init_instance (const char *muhome)
 		return FALSE;
 
 	err = NULL;
-	store = mu_store_new_read_only
+	store = mu_store_new_readable
 		(mu_runtime_path(MU_RUNTIME_PATH_XAPIANDB),
 		 &err);
 	if (!store)
@@ -144,7 +145,7 @@ SCM_DEFINE_PUBLIC (mu_initialize, "mu:initialize", 0, 1, 0,
 		   "Initialize mu - needed before you call any of the other "
 		   "functions. Optionally, you can provide MUHOME which should be an "
 		   "absolute path to your mu home directory "
-		   "-- typically, the default, ~/.mu, should be just fine.")
+		   "-- typically, the default, ~/.cache/mu, should be just fine.")
 #define FUNC_NAME s_mu_initialize
 {
 	char *muhome;
