@@ -1,7 +1,5 @@
-/* -*-mode: c; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-*/
-
 /*
-** Copyright (C) 2008-2013 Dirk-Jan C. Binnema <djcb@djcbsoftware.nl>
+** Copyright (C) 2008-2020 Dirk-Jan C. Binnema <djcb@djcbsoftware.nl>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -24,8 +22,8 @@
 
 #include <stdlib.h>
 #include <glib.h>
-#include <mu-util.h> /* for MuResult */
-#include <mu-store.h>
+#include <utils/mu-util.h>
+#include <mu-store.hh>
 
 G_BEGIN_DECLS
 
@@ -37,15 +35,15 @@ struct _MuIndexStats {
 	unsigned _processed;     /* number of msgs processed or counted */
 	unsigned _updated;       /* number of msgs new or updated */
 	unsigned _cleaned_up;    /* number of msgs cleaned up */
-	unsigned _uptodate;      /* number of msgs already uptodate */
+	unsigned _uptodate;      /* number of msgs already up-to-date */
 };
 typedef struct _MuIndexStats MuIndexStats;
 
 /**
- * create a new MuIndex instance. NOTE(1): the database does not have
+ * create a new MuIndex instance. NOTE: the database does not have
  * to exist yet, but the directory must already exist; NOTE(2): before
  * doing anything with the returned Index object, make sure you haved
- * called g_type_init, and mu_msg_init somewhere in your code.
+ * called mu_msg_init somewhere in your code.
  *
  * @param store a writable MuStore object
  * @param err to receive error or NULL; there are only errors when this
@@ -77,16 +75,6 @@ void mu_index_destroy (MuIndex *index);
 void mu_index_set_max_msg_size (MuIndex *index, guint max_size);
 
 
-/**
- * change batch size for Xapian store transaction (see
- * 'mu_store_set_batch_size')
- *
- * @param index a mu index object
- * @param max_size the batch size, or 0 to reset to the default
- */
-void mu_index_set_xbatch_size (MuIndex *index, guint xbatchsize);
-
-
 /**
  * callback function for mu_index_(run|stats|cleanup), for each message
  *
@@ -106,7 +94,7 @@ typedef MuError (*MuIndexMsgCallback) (MuIndexStats* stats, void *user_data);
  * @param enter did we enter (TRUE) or leave(FALSE) the dir?
  * @param user_data pointer to user data
  *
- * @return  MU_OK to contiue, MU_STOP to stopd or MU_ERROR in
+ * @return  MU_OK to continue, MU_STOP to stopd or MU_ERROR in
  * case of some error.
  */
 typedef MuError (*MuIndexDirCallback) (const char* path, gboolean enter,
@@ -116,9 +104,10 @@ typedef MuError (*MuIndexDirCallback) (const char* path, gboolean enter,
  * start the indexing process
  *
  * @param index a valid MuIndex instance
- * @param path the path to index. This must be an absolute path
  * @param force if != 0, force re-indexing already index messages; this is
  *         obviously a lot slower than only indexing new/changed messages
+ * @param lazycheck whether ignore subdirectoryies that have up-to-date
+ * timestamps.
  * @param stats a structure with some statistics about the results;
  * note that this function does *not* reset the struct values to allow
  * for cumulative stats from multiple calls. If needed, you can use
@@ -127,22 +116,22 @@ typedef MuError (*MuIndexDirCallback) (const char* path, gboolean enter,
  * @param cb_dir a callback function called for every dir entered/left or NULL
  * @param user_data a user pointer that will be passed to the callback function
  *
- * @return MU_OK if the stats gathering was completed succesfully,
+ * @return MU_OK if the stats gathering was completed successfully,
  * MU_STOP if the user stopped or MU_ERROR in
  * case of some error.
  */
-MuError mu_index_run (MuIndex *index, const char *path, gboolean force,
-		      MuIndexStats *stats, MuIndexMsgCallback msg_cb,
+MuError mu_index_run (MuIndex *index, gboolean force,
+		      gboolean lazycheck, MuIndexStats *stats,
+		      MuIndexMsgCallback msg_cb,
 		      MuIndexDirCallback dir_cb, void *user_data);
 
 /**
- * gather some statistics about the Maildir; this is usually much faster
- * than mu_index_run, and can thus be used to provide some information to the user
+ * gather some statistics about the Maildir; this is usually much faster than
+ * mu_index_run, and can thus be used to provide some information to the user
  * note though that the statistics may be different from the reality that
  * mu_index_run sees, when there are updates in the Maildir
  *
  * @param index a valid MuIndex instance
- * @param path the path to get stats for; this must be an absolute path
  * @param stats a structure with some statistics about the results;
  * note that this function does *not* reset the struct values to allow
  * for cumulative stats from multiple calls. If needed, you can use
@@ -151,16 +140,14 @@ MuError mu_index_run (MuIndex *index, const char *path, gboolean force,
  * @param dir_cb a callback function which will be called for every dir or NULL
  * @param user_data a user pointer that will be passed to the callback function
  * xb
- * @return MU_OK if the stats gathering was completed succesfully,
+ * @return MU_OK if the stats gathering was completed successfully,
  * MU_STOP if the user stopped or MU_ERROR in
  * case of some error.
  */
-MuError mu_index_stats (MuIndex *index, const char *path, MuIndexStats *stats,
+MuError mu_index_stats (MuIndex *index, MuIndexStats *stats,
 			MuIndexMsgCallback msg_cb, MuIndexDirCallback dir_cb,
 			void *user_data);
 
-
-
 /**
  * callback function called for each message
  *
@@ -184,7 +171,7 @@ typedef MuError (*MuIndexCleanupDeleteCallback) (MuIndexStats *stats,
  * @param user_data a user pointer that will be passed to the callback function
  * @param err to receive error info or NULL. err->code is MuError value
  *
- * @return MU_OK if the stats gathering was completed succesfully,
+ * @return MU_OK if the stats gathering was completed successfully,
  * MU_STOP if the user stopped or MU_ERROR in
  * case of some error.
  */
