@@ -83,6 +83,24 @@ MuMsgIter *mu_msg_iter_new (XapianEnquire *enq,
  */
 gboolean  mu_msg_iter_next  (MuMsgIter *iter);
 
+/**
+ * Does this iterator point to the first item?
+ *
+ * @param iter a valid MuMsgIter iterator
+ *
+ * @return TRUE or FALSE
+ */
+gboolean  mu_msg_iter_is_first  (MuMsgIter *iter);
+
+/**
+ * Does this iterator point to the last item?
+ *
+ * @param iter a valid MuMsgIter iterator
+ *
+ * @return TRUE or FALSE
+ */
+gboolean  mu_msg_iter_is_last  (MuMsgIter *iter);
+
 
 /**
  * reset the iterator to the beginning
@@ -124,8 +142,6 @@ void		 mu_msg_iter_destroy           (MuMsgIter *iter);
 MuMsg* mu_msg_iter_get_msg_floating (MuMsgIter *iter)
           G_GNUC_MALLOC G_GNUC_WARN_UNUSED_RESULT;
 
-
-
 /**
  * Provide a preferred_hash, which is a hashtable msgid->docid to
  * indicate the messages which should /not/ be seen as duplicates.
@@ -136,8 +152,6 @@ MuMsg* mu_msg_iter_get_msg_floating (MuMsgIter *iter)
  */
 void mu_msg_iter_set_preferred (MuMsgIter *iter, GHashTable *preferred_hash);
 
-
-
 /**
  * get the document id for the current message
  *
@@ -153,7 +167,7 @@ guint    mu_msg_iter_get_docid         (MuMsgIter *iter);
  *
  * @param iter a valid MuMsgIter iterator
  *
- * @return TRUE if it worked, FALSE otherwsie.
+ * @return TRUE if it worked, FALSE otherwise.
  */
 gboolean mu_msg_iter_calculate_threads (MuMsgIter *iter);
 
@@ -163,14 +177,15 @@ enum _MuMsgIterThreadProp {
 
 	MU_MSG_ITER_THREAD_PROP_ROOT           = 1 << 0,
 	MU_MSG_ITER_THREAD_PROP_FIRST_CHILD    = 1 << 1,
-	MU_MSG_ITER_THREAD_PROP_EMPTY_PARENT   = 1 << 2,
-	MU_MSG_ITER_THREAD_PROP_DUP            = 1 << 3,
-	MU_MSG_ITER_THREAD_PROP_HAS_CHILD      = 1 << 4
+	MU_MSG_ITER_THREAD_PROP_LAST_CHILD     = 1 << 2,
+	MU_MSG_ITER_THREAD_PROP_EMPTY_PARENT   = 1 << 3,
+	MU_MSG_ITER_THREAD_PROP_DUP            = 1 << 4,
+	MU_MSG_ITER_THREAD_PROP_HAS_CHILD      = 1 << 5
 };
 typedef guint8 MuMsgIterThreadProp;
 
 struct _MuMsgIterThreadInfo {
-	gchar *threadpath; /* a string decribing the thread-path in
+	gchar *threadpath; /* a string describing the thread-path in
 			    * such a way that we can sort by this
 			    * string to get the right order. */
 	guint level;       /* thread-depth -- [0...] */
@@ -189,8 +204,6 @@ typedef struct _MuMsgIterThreadInfo MuMsgIterThreadInfo;
  */
 const MuMsgIterThreadInfo* mu_msg_iter_get_thread_info (MuMsgIter *iter);
 
-
-
 /**
  * get the message-id for this message
  *
