@@ -25,8 +25,8 @@
 #include <string.h>
 #include <unistd.h>
 
-#include "mu-util.h"
-#include "mu-str.h"
+#include "utils/mu-util.h"
+#include "utils/mu-str.h"
 #include "mu-msg-priv.h"
 #include "mu-msg-part.h"
 
@@ -59,6 +59,10 @@ get_mime_object_at_index (MuMsg *msg, MuMsgOptions opts, unsigned index)
 	ddata.mime_obj  = NULL;
 	ddata.index     = index;
 
+	/* wipe out some irrelevant options */
+	opts &= ~MU_MSG_OPTION_VERIFY;
+	opts &= ~MU_MSG_OPTION_EXTRACT_IMAGES;
+
 	mu_msg_part_foreach (msg, opts,
 			     (MuMsgPartForeachFunc)do_it_with_index,
 			     &ddata);
@@ -105,41 +109,49 @@ get_matching_part_index (MuMsg *msg, MuMsgOptions opts,
 static void
 accumulate_text_message (MuMsg *msg, MuMsgPart *part, GString **gstrp)
 {
-	const gchar *str;
-	char *adrs;
-	GMimeMessage *mimemsg;
-	InternetAddressList *addresses;
+	const gchar		*str;
+	char			*adrs;
+	GMimeMessage		*mimemsg;
+	InternetAddressList	*addresses;
 
 	/* put sender, recipients and subject in the string, so they
 	 * can be indexed as well */
-	mimemsg = GMIME_MESSAGE (part->data);
-	str = g_mime_message_get_sender (mimemsg);
+	mimemsg	  = GMIME_MESSAGE (part->data);
+	addresses = g_mime_message_get_addresses (mimemsg, GMIME_ADDRESS_TYPE_FROM);
+	adrs	  = internet_address_list_to_string (addresses, NULL, FALSE);
+
 	g_string_append_printf
-		(*gstrp, "%s%s", str ? str : "", str ? "\n" : "");
+		(*gstrp, "%s%s", adrs ? adrs : "", adrs ? "\n" : "");
+	g_free (adrs);
+
 	str = g_mime_message_get_subject (mimemsg);
 	g_string_append_printf
 		(*gstrp, "%s%s", str ? str : "", str ? "\n" : "");
-		addresses = g_mime_message_get_all_recipients (mimemsg);
-		adrs = internet_address_list_to_string (addresses, FALSE);
-		g_object_unref (addresses);
-		g_string_append_printf
-			(*gstrp, "%s%s", adrs ? adrs : "", adrs ? "\n" : "");
-		g_free (adrs);
+
+	addresses = g_mime_message_get_all_recipients (mimemsg);
+	adrs	  = internet_address_list_to_string (addresses, NULL, FALSE);
+	g_object_unref (addresses);
+
+	g_string_append_printf
+		(*gstrp, "%s%s", adrs ? adrs : "", adrs ? "\n" : "");
+	g_free (adrs);
 }
 
 static void
 accumulate_text_part (MuMsg *msg, MuMsgPart *part, GString **gstrp)
 {
-	GMimeContentType *ctype;
-	gboolean err;
-	char *txt;
+	GMimeContentType	*ctype;
+	gboolean		 err;
+	char			*txt;
+
 	ctype = g_mime_object_get_content_type ((GMimeObject*)part->data);
 	if (!g_mime_content_type_is_type (ctype, "text", "plain"))
 		return; /* not plain text */
-	txt = mu_msg_mime_part_to_string
-		((GMimePart*)part->data, &err);
+
+	txt = mu_msg_mime_part_to_string((GMimePart*)part->data, &err);
 	if (txt)
 		g_string_append (*gstrp, txt);
+
 	g_free (txt);
 }
 
@@ -225,7 +237,7 @@ get_part_size (GMimePart *part)
 	GMimeDataWrapper *wrapper;
 	GMimeStream *stream;
 
-	wrapper = g_mime_part_get_content_object (part);
+	wrapper = g_mime_part_get_content (part);
 	if (!GMIME_IS_DATA_WRAPPER(wrapper))
 		return -1;
 
@@ -461,6 +473,7 @@ looks_like_text_body_part (GMimeContentType *ctype)
 		const char *subtype;
 	} types[] = {
 		{ "text", "plain" },
+		{ "text", "x-markdown" },
 		{ "text", "x-diff"  },
 		{ "text", "x-patch" },
 		{ "application", "x-patch"}
@@ -476,6 +489,27 @@ looks_like_text_body_part (GMimeContentType *ctype)
 }
 
 
+static MuMsgPartSigStatusReport*
+copy_status_report_maybe (GObject *obj)
+{
+	MuMsgPartSigStatusReport *report, *copy;
+
+	report = g_object_get_data (obj, SIG_STATUS_REPORT);
+	if (!report)
+		return NULL; /* nothing to copy */
+
+	copy = g_slice_new0(MuMsgPartSigStatusReport);
+	copy->verdict = report->verdict;
+
+	if (report->report)
+		copy->report  = g_strdup (report->report);
+	if (report->signers)
+		copy->signers = g_strdup (report->signers);
+
+	return copy;
+}
+
+
 
 /* call 'func' with information about this MIME-part */
 static gboolean
@@ -515,10 +549,7 @@ handle_part (MuMsg *msg, GMimePart *part, GMimeObject *parent,
 	if (g_ascii_strcasecmp (msgpart.subtype, "pgp-signature") == 0 ||
 	    decrypted) {
 		msgpart.sig_status_report =
-			(MuMsgPartSigStatusReport*)
-			g_object_get_data (G_OBJECT(parent),
-					   SIG_STATUS_REPORT);
-
+			copy_status_report_maybe (G_OBJECT(parent));
 		if (msgpart.sig_status_report)
 			msgpart.part_type |= MU_MSG_PART_TYPE_SIGNED;
 	}
@@ -528,6 +559,8 @@ handle_part (MuMsg *msg, GMimePart *part, GMimeObject *parent,
 
 	func (msg, &msgpart, user_data);
 
+	mu_msg_part_sig_status_report_destroy (msgpart.sig_status_report);
+
 	return TRUE;
 }
 
@@ -665,7 +698,7 @@ write_part_to_fd (GMimePart *part, int fd, GError **err)
 	}
 	g_mime_stream_fs_set_owner (GMIME_STREAM_FS(stream), FALSE);
 
-	wrapper = g_mime_part_get_content_object (part);
+	wrapper = g_mime_part_get_content (part);
 	if (!GMIME_IS_DATA_WRAPPER(wrapper)) {
 		g_set_error (err, MU_ERROR_DOMAIN, MU_ERROR_GMIME,
 			     "failed to create wrapper");
@@ -694,7 +727,7 @@ static gboolean
 write_object_to_fd (GMimeObject *obj, int fd, GError **err)
 {
 	gchar *str;
-	str = g_mime_object_to_string (obj);
+	str = g_mime_object_to_string (obj, NULL);
 
 	if (!str) {
 		g_set_error (err, MU_ERROR_DOMAIN, MU_ERROR_GMIME,
@@ -743,10 +776,6 @@ save_object (GMimeObject *obj, MuMsgOptions opts, const char *fullpath,
 	else
 		rv = write_object_to_fd (obj, fd, err);
 
-	/* Unref it since it was referenced earlier by
-	 * get_mime_object_at_index */
-	g_object_unref (obj);
-
 	if (close (fd) != 0 && !err) { /* don't write on top of old err */
 		g_set_error (err, MU_ERROR_DOMAIN, MU_ERROR_FILE,
 			     "could not close '%s': %s",
@@ -830,15 +859,18 @@ gboolean
 mu_msg_part_save (MuMsg *msg, MuMsgOptions opts,
 		  const char *fullpath, guint partidx, GError **err)
 {
-	GMimeObject *part;
+	gboolean	 rv;
+	GMimeObject	*part;
 
 	g_return_val_if_fail (msg, FALSE);
 	g_return_val_if_fail (fullpath, FALSE);
 	g_return_val_if_fail (!((opts & MU_MSG_OPTION_OVERWRITE) &&
 				(opts & MU_MSG_OPTION_USE_EXISTING)), FALSE);
 
+	rv = FALSE;
+
 	if (!mu_msg_load_msg_file (msg, err))
-		return FALSE;
+		return rv;
 
 	part = get_mime_object_at_index (msg, opts, partidx);
 
@@ -847,21 +879,20 @@ mu_msg_part_save (MuMsg *msg, MuMsgOptions opts,
 		part = (GMimeObject*)g_mime_message_part_get_message
 			(GMIME_MESSAGE_PART (part));
 
-	if (!part) {
+	if (!part)
 		g_set_error (err, MU_ERROR_DOMAIN, MU_ERROR_GMIME,
 			     "part %u does not exist", partidx);
-		return FALSE;
-	}
-	if (!GMIME_IS_PART(part) && !GMIME_IS_MESSAGE(part)) {
+	else if (!GMIME_IS_PART(part) && !GMIME_IS_MESSAGE(part))
 		g_set_error (err, MU_ERROR_DOMAIN, MU_ERROR_GMIME,
 			     "unexpected type %s for part %u",
 			     G_OBJECT_TYPE_NAME((GObject*)part),
 			     partidx);
-		return FALSE;
-	}
+	else
+		rv = save_object (part, opts, fullpath, err);
 
+	g_clear_object(&part);
 
-	return save_object (part, opts, fullpath, err);
+	return rv;
 }
 
 
