@@ -19,7 +19,7 @@
 #include <string.h>
 #include <ctype.h>
 
-#include "mu-str.h"
+#include "utils/mu-str.h"
 #include "mu-msg.h"
 #include "mu-msg-iter.h"
 #include "mu-msg-part.h"
@@ -46,9 +46,6 @@ append_sexp_attr_list (GString *gstr, const char* elm, const GSList *lst)
 	g_string_append (gstr, ")\n");
 }
 
-
-
-
 static void
 append_sexp_attr (GString *gstr, const char* elm, const char *str)
 {
@@ -71,7 +68,6 @@ append_sexp_attr (GString *gstr, const char* elm, const char *str)
 	g_free (esc);
 }
 
-
 static void
 append_sexp_body_attr (GString *gstr, const char* elm, const char *str)
 {
@@ -86,11 +82,6 @@ append_sexp_body_attr (GString *gstr, const char* elm, const char *str)
 	g_free (esc);
 }
 
-
-
-
-
-
 struct _ContactData {
 	gboolean from, to, cc, bcc, reply_to;
 	GString *gstr;
@@ -99,25 +90,26 @@ struct _ContactData {
 typedef struct _ContactData ContactData;
 
 static gchar*
-get_name_addr_pair (MuMsgContact *c)
+get_name_email_pair (MuMsgContact *c)
 {
-	gchar *name, *addr, *pair;
+	gchar *name, *email, *pair;
 
-	name = (char*)mu_msg_contact_name(c);
-	addr = (char*)mu_msg_contact_address(c);
+	name  = (char*)mu_msg_contact_name(c);
+	email = (char*)mu_msg_contact_email(c);
 
-	name = name ? mu_str_escape_c_literal (name, TRUE) : NULL;
-	addr = addr ? mu_str_escape_c_literal (addr, TRUE) : NULL;
+	name  = name ? mu_str_escape_c_literal (name, TRUE) : NULL;
+	email = email ? mu_str_escape_c_literal (email, TRUE) : NULL;
 
 	pair = g_strdup_printf ("(%s . %s)",
 				name ? name : "nil",
-				addr ? addr : "nil");
+				email ? email : "nil");
 	g_free (name);
-	g_free (addr);
+	g_free (email);
 
 	return pair;
 }
 
+
 static void
 add_prefix_maybe (GString *gstr, gboolean *field, const char *prefix)
 {
@@ -164,13 +156,43 @@ each_contact (MuMsgContact *c, ContactData *cdata)
 
 	cdata->prev_ctype = ctype;
 
-	pair = get_name_addr_pair (c);
+	pair = get_name_email_pair (c);
 	g_string_append (cdata->gstr, pair);
 	g_free (pair);
 
 	return TRUE;
 }
 
+static void
+maybe_append_list_post (GString *gstr, MuMsg *msg)
+{
+	/* some mailing lists do not set the reply-to; see pull #1278. So for
+	 * those cases, check the List-Post address and use that instead */
+
+	GMatchInfo	*minfo;
+	GRegex		*rx;
+	const char*	 list_post;
+
+	list_post = mu_msg_get_header (msg, "List-Post");
+	if (!list_post)
+		return;
+
+	rx = g_regex_new ("<?mailto:([a-z0-9!@#$%&'*+-/=?^_`{|}~]+)>?",
+                          G_REGEX_CASELESS, (GRegexMatchFlags)0, NULL);
+	g_return_if_fail(rx);
+
+	if (g_regex_match (rx, list_post, 0, &minfo)) {
+		char	*addr;
+		addr = g_match_info_fetch (minfo, 1);
+		g_string_append_printf (gstr,"\t:list-post ((nil . \"%s\"))\n", addr);
+		g_free(addr);
+	}
+
+	g_match_info_free (minfo);
+	g_regex_unref (rx);
+}
+
+
 
 static void
 append_sexp_contacts (GString *gstr, MuMsg *msg)
@@ -184,9 +206,10 @@ append_sexp_contacts (GString *gstr, MuMsg *msg)
 
 	mu_msg_contact_foreach (msg, (MuMsgContactForeachFunc)each_contact,
 				&cdata);
-
 	if (cdata.from || cdata.to || cdata.cc || cdata.bcc || cdata.reply_to)
 		gstr = g_string_append (gstr, ")\n");
+
+        maybe_append_list_post (gstr, msg);
 }
 
 struct _FlagData {
@@ -368,8 +391,9 @@ get_part_type_string (MuMsgPartType ptype)
 static void
 each_part (MuMsg *msg, MuMsgPart *part, PartInfo *pinfo)
 {
-	char	*name, *encname, *tmp, *parttype;
-	char	*tmpfile, *cid, *verdict;
+	char		*name, *encname, *tmp, *parttype;
+	char		*tmpfile, *cidesc, *verdict;
+	const char	*cid;
 
 	name     = mu_msg_part_get_filename (part, TRUE);
 	encname  = name ?
@@ -380,9 +404,9 @@ each_part (MuMsg *msg, MuMsgPart *part, PartInfo *pinfo)
 	tmpfile  = get_temp_file_maybe (msg, part, pinfo->opts);
 	parttype = get_part_type_string (part->part_type);
 	verdict  = sig_verdict (part);
-	cid      = mu_str_escape_c_literal(mu_msg_part_get_content_id(part),
-					   TRUE);
 
+	cid    = mu_msg_part_get_content_id(part);
+	cidesc = cid ? mu_str_escape_c_literal(cid, TRUE) : NULL;
 
 	tmp = g_strdup_printf
 		("%s(:index %d :name %s :mime-type \"%s/%s\"%s%s "
@@ -396,7 +420,7 @@ each_part (MuMsg *msg, MuMsgPart *part, PartInfo *pinfo)
 		 tmpfile ? " :temp" : "", tmpfile ? tmpfile : "",
 		 parttype,
 		 mu_msg_part_maybe_attachment (part) ? "t" : "nil",
-		 cid ? " :cid" : "", cid ? cid : "",
+		 cidesc ? " :cid" : "", cidesc ? cidesc : "",
 		 (int)part->size,
 		 verdict,
 		 dec_verdict (part));
@@ -405,7 +429,7 @@ each_part (MuMsg *msg, MuMsgPart *part, PartInfo *pinfo)
 	g_free (tmpfile);
 	g_free (parttype);
 	g_free (verdict);
-	g_free (cid);
+	g_free (cidesc);
 
 	g_free (pinfo->parts);
 	pinfo->parts = tmp;
@@ -433,11 +457,13 @@ static void
 append_sexp_thread_info (GString *gstr, const MuMsgIterThreadInfo *ti)
 {
 	g_string_append_printf
-		(gstr, "\t:thread (:path \"%s\" :level %u%s%s%s%s)\n",
+		(gstr, "\t:thread (:path \"%s\" :level %u%s%s%s%s%s)\n",
 		 ti->threadpath,
 		 ti->level,
 		 ti->prop & MU_MSG_ITER_THREAD_PROP_FIRST_CHILD  ?
 		 " :first-child t" : "",
+		 ti->prop & MU_MSG_ITER_THREAD_PROP_LAST_CHILD   ?
+		 " :last-child t" : "",
 		 ti->prop & MU_MSG_ITER_THREAD_PROP_EMPTY_PARENT ?
 		 " :empty-parent t" : "",
 		 ti->prop & MU_MSG_ITER_THREAD_PROP_DUP          ?
@@ -480,7 +506,7 @@ append_message_file_parts (GString *gstr, MuMsg *msg, MuMsgOptions opts)
 
 	if (!mu_msg_load_msg_file (msg, &err)) {
 		g_warning ("failed to load message file: %s",
-			   err ? err->message : "some error occured");
+			   err ? err->message : "some error occurred");
 		g_clear_error (&err);
 		return;
 	}
@@ -554,7 +580,6 @@ append_sexp_tags (GString *gstr, MuMsg *msg)
 	g_string_free (tagstr, TRUE);
 }
 
-
 char*
 mu_msg_to_sexp (MuMsg *msg, unsigned docid, const MuMsgIterThreadInfo *ti,
 		MuMsgOptions opts)
@@ -577,9 +602,8 @@ mu_msg_to_sexp (MuMsg *msg, unsigned docid, const MuMsgIterThreadInfo *ti,
 
 	append_sexp_attr (gstr, "subject", mu_msg_get_subject (msg));
 
-	/* in the no-headers-only case (see below) we get a more
-	 * complete list of contacts, so no need to get them here if
-	 * that's the case */
+	/* in the no-headers-only case (see below) we get a more complete list
+	 * of contacts, so no need to get them here if that's the case */
 	if (opts & MU_MSG_OPTION_HEADERS_ONLY)
 		append_sexp_contacts (gstr, msg);
 
