@@ -30,9 +30,7 @@
 
 #include "mu-msg-priv.h" /* include before mu-msg.h */
 #include "mu-msg.h"
-
-#include "mu-util.h"
-#include "mu-str.h"
+#include "utils/mu-str.h"
 
 #include "mu-maildir.h"
 
@@ -48,12 +46,7 @@ gmime_init (void)
 {
 	g_return_if_fail (!_gmime_initialized);
 
-#ifdef GMIME_ENABLE_RFC2047_WORKAROUNDS
-	g_mime_init(GMIME_ENABLE_RFC2047_WORKAROUNDS);
-#else
-	g_mime_init(0);
-#endif /* GMIME_ENABLE_RFC2047_WORKAROUNDS */
-
+	g_mime_init();
 	_gmime_initialized = TRUE;
 }
 
@@ -66,7 +59,6 @@ gmime_uninit (void)
 	_gmime_initialized = FALSE;
 }
 
-
 static MuMsg*
 msg_new (void)
 {
@@ -102,7 +94,6 @@ mu_msg_new_from_file (const char *path, const char *mdir,
 	return self;
 }
 
-
 MuMsg*
 mu_msg_new_from_doc (XapianDocument *doc, GError **err)
 {
@@ -126,7 +117,6 @@ mu_msg_new_from_doc (XapianDocument *doc, GError **err)
 	return self;
 }
 
-
 static void
 mu_msg_destroy (MuMsg *self)
 {
@@ -146,7 +136,6 @@ mu_msg_destroy (MuMsg *self)
 	g_slice_free (MuMsg, self);
 }
 
-
 MuMsg*
 mu_msg_ref (MuMsg *self)
 {
@@ -185,7 +174,6 @@ free_later_lst (MuMsg *self, GSList *lst)
 	return lst;
 }
 
-
 /* use this instead of mu_msg_get_path so we don't get into infinite
  * regress...*/
 static const char*
@@ -214,7 +202,6 @@ get_path (MuMsg *self)
 	return free_later_str (self, val);
 }
 
-
 /* for some data, we need to read the message file from disk */
 gboolean
 mu_msg_load_msg_file (MuMsg *self, GError **err)
@@ -237,7 +224,6 @@ mu_msg_load_msg_file (MuMsg *self, GError **err)
 	return  (self->_file != NULL);
 }
 
-
 void
 mu_msg_unload_msg_file (MuMsg *msg)
 {
@@ -247,7 +233,6 @@ mu_msg_unload_msg_file (MuMsg *msg)
 	msg->_file = NULL;
 }
 
-
 static const GSList*
 get_str_list_field (MuMsg *self, MuMsgFieldId mfid)
 {
@@ -268,7 +253,6 @@ get_str_list_field (MuMsg *self, MuMsgFieldId mfid)
 	return free_later_lst (self, val);
 }
 
-
 static const char*
 get_str_field (MuMsg *self, MuMsgFieldId mfid)
 {
@@ -293,7 +277,6 @@ get_str_field (MuMsg *self, MuMsgFieldId mfid)
 	return do_free ? free_later_str (self, val) : val;
 }
 
-
 static gint64
 get_num_field (MuMsg *self, MuMsgFieldId mfid)
 {
@@ -313,7 +296,6 @@ get_num_field (MuMsg *self, MuMsgFieldId mfid)
 	return val;
 }
 
-
 const char*
 mu_msg_get_header (MuMsg *self, const char *header)
 {
@@ -329,7 +311,6 @@ mu_msg_get_header (MuMsg *self, const char *header)
 		(self, mu_msg_file_get_header (self->_file, header));
 }
 
-
 time_t
 mu_msg_get_timestamp (MuMsg *self)
 {
@@ -348,7 +329,6 @@ mu_msg_get_timestamp (MuMsg *self)
 	return statbuf.st_mtime;
 }
 
-
 const char*
 mu_msg_get_path (MuMsg *self)
 {
@@ -356,7 +336,6 @@ mu_msg_get_path (MuMsg *self)
 	return get_str_field (self, MU_MSG_FIELD_ID_PATH);
 }
 
-
 const char*
 mu_msg_get_subject  (MuMsg *self)
 {
@@ -371,8 +350,6 @@ mu_msg_get_msgid  (MuMsg *self)
 	return get_str_field (self, MU_MSG_FIELD_ID_MSGID);
 }
 
-
-
 const char*
 mu_msg_get_mailing_list (MuMsg *self)
 {
@@ -385,14 +362,13 @@ mu_msg_get_mailing_list (MuMsg *self)
 	if (!ml)
 		return NULL;
 
-	decml = g_mime_utils_header_decode_text (ml);
+	decml = g_mime_utils_header_decode_text (NULL, ml);
 	if (!decml)
 		return NULL;
 
 	return free_later_str (self, decml);
 }
 
-
 const char*
 mu_msg_get_maildir (MuMsg *self)
 {
@@ -400,7 +376,6 @@ mu_msg_get_maildir (MuMsg *self)
 	return get_str_field (self, MU_MSG_FIELD_ID_MAILDIR);
 }
 
-
 const char*
 mu_msg_get_from (MuMsg *self)
 {
@@ -408,7 +383,6 @@ mu_msg_get_from (MuMsg *self)
 	return get_str_field (self, MU_MSG_FIELD_ID_FROM);
 }
 
-
 const char*
 mu_msg_get_to (MuMsg *self)
 {
@@ -423,7 +397,6 @@ mu_msg_get_cc (MuMsg *self)
 	return get_str_field (self, MU_MSG_FIELD_ID_CC);
 }
 
-
 const char*
 mu_msg_get_bcc (MuMsg *self)
 {
@@ -431,7 +404,6 @@ mu_msg_get_bcc (MuMsg *self)
 	return get_str_field (self, MU_MSG_FIELD_ID_BCC);
 }
 
-
 time_t
 mu_msg_get_date (MuMsg *self)
 {
@@ -439,7 +411,6 @@ mu_msg_get_date (MuMsg *self)
 	return (time_t)get_num_field (self, MU_MSG_FIELD_ID_DATE);
 }
 
-
 MuFlags
 mu_msg_get_flags (MuMsg *self)
 {
@@ -454,7 +425,6 @@ mu_msg_get_size (MuMsg *self)
 	return (size_t)get_num_field (self, MU_MSG_FIELD_ID_SIZE);
 }
 
-
 MuMsgPrio
 mu_msg_get_prio (MuMsg *self)
 {
@@ -462,45 +432,39 @@ mu_msg_get_prio (MuMsg *self)
 	return (MuMsgPrio)get_num_field (self, MU_MSG_FIELD_ID_PRIO);
 }
 
-
-
 struct _BodyData {
 	GString *gstr;
 	gboolean want_html;
 };
 typedef struct _BodyData BodyData;
 
-
 static void
 accumulate_body (MuMsg *msg, MuMsgPart *mpart, BodyData *bdata)
 {
-	char *txt;
-	gboolean err;
+	char		*txt;
+	GMimePart       *mimepart;
+	gboolean	 has_err, is_plain, is_html;
 
 	if (!GMIME_IS_PART(mpart->data))
 		return;
+	if (mpart->part_type &	MU_MSG_PART_TYPE_ATTACHMENT)
+		return;
 
-	txt = NULL;
-	err = TRUE;
+	mimepart = (GMimePart*)mpart->data;
+	is_html  = mpart->part_type &	MU_MSG_PART_TYPE_TEXT_HTML;
+	is_plain = mpart->part_type &	MU_MSG_PART_TYPE_TEXT_PLAIN;
 
-	/* text-like attachments are included when in text-mode */
+	txt	    = NULL;
+	has_err	    = TRUE;
+	if ((bdata->want_html && is_html) || (!bdata->want_html && is_plain))
+		txt = mu_msg_mime_part_to_string (mimepart, &has_err);
 
-	if (!bdata->want_html &&
-	    (mpart->part_type & MU_MSG_PART_TYPE_TEXT_PLAIN))
-		txt = mu_msg_mime_part_to_string (
-			(GMimePart*)mpart->data, &err);
-	else if (!(mpart->part_type & MU_MSG_PART_TYPE_ATTACHMENT) &&
-		 bdata->want_html &&
-		 (mpart->part_type & MU_MSG_PART_TYPE_TEXT_HTML))
-		txt = mu_msg_mime_part_to_string (
-			(GMimePart*)mpart->data, &err);
-	if (!err && txt)
+	if (!has_err && txt)
 		bdata->gstr = g_string_append (bdata->gstr, txt);
 
 	g_free (txt);
 }
 
-
 static char*
 get_body (MuMsg *self, MuMsgOptions opts, gboolean want_html)
 {
@@ -509,6 +473,10 @@ get_body (MuMsg *self, MuMsgOptions opts, gboolean want_html)
 	bdata.want_html = want_html;
 	bdata.gstr	= g_string_sized_new (4096);
 
+	/* wipe out some irrelevant options */
+	opts &= ~MU_MSG_OPTION_VERIFY;
+	opts &= ~MU_MSG_OPTION_EXTRACT_IMAGES;
+
 	mu_msg_part_foreach (self, opts,
 			     (MuMsgPartForeachFunc)accumulate_body,
 			     &bdata);
@@ -520,7 +488,6 @@ get_body (MuMsg *self, MuMsgOptions opts, gboolean want_html)
 		return g_string_free (bdata.gstr, FALSE);
 }
 
-
 typedef struct {
 	GMimeContentType	*ctype;
 	gboolean		 want_html;
@@ -551,7 +518,6 @@ find_content_type (MuMsg *msg, MuMsgPart *mpart, ContentTypeData *cdata)
 			GMIME_OBJECT(wanted));
 }
 
-
 static const GSList*
 get_content_type_parameters (MuMsg *self, MuMsgOptions opts, gboolean want_html)
 {
@@ -560,32 +526,36 @@ get_content_type_parameters (MuMsg *self, MuMsgOptions opts, gboolean want_html)
 	cdata.want_html = want_html;
 	cdata.ctype	= NULL;
 
+	/* wipe out some irrelevant options */
+	opts &= ~MU_MSG_OPTION_VERIFY;
+	opts &= ~MU_MSG_OPTION_EXTRACT_IMAGES;
+
 	mu_msg_part_foreach (self, opts,
 			     (MuMsgPartForeachFunc)find_content_type,
 			     &cdata);
 
 	if (cdata.ctype) {
 
-		GSList			*paramlist;
+		GSList			*gslist;
+		GMimeParamList		*paramlist;
 		const GMimeParam	*param;
+		int i, len;
 
-		paramlist = NULL;
-		param	  = g_mime_content_type_get_params (cdata.ctype);
+		gslist	  = NULL;
+		paramlist = g_mime_content_type_get_parameters (cdata.ctype);
+		len	  = g_mime_param_list_length (paramlist);
 
-		for (; param; param = param->next) {
-			paramlist = g_slist_prepend (paramlist,
-						     g_strdup (param->name));
-
-			paramlist = g_slist_prepend (paramlist,
-						     g_strdup (param->value));
+		for (i = 0; i < len; ++i) {
+			param = g_mime_param_list_get_parameter_at (paramlist, i);
+			gslist = g_slist_prepend (gslist, g_strdup (param->name));
+			gslist = g_slist_prepend (gslist, g_strdup (param->value));
 		}
 
-		return free_later_lst(self, g_slist_reverse (paramlist));
+		return free_later_lst (self, g_slist_reverse (gslist));
 	}
 	return NULL;
 }
 
-
 const GSList*
 mu_msg_get_body_text_content_type_parameters (MuMsg *self, MuMsgOptions opts)
 {
@@ -593,7 +563,6 @@ mu_msg_get_body_text_content_type_parameters (MuMsg *self, MuMsgOptions opts)
 	return get_content_type_parameters(self, opts, FALSE);
 }
 
-
 const char*
 mu_msg_get_body_html (MuMsg *self, MuMsgOptions opts)
 {
@@ -601,8 +570,6 @@ mu_msg_get_body_html (MuMsg *self, MuMsgOptions opts)
 	return free_later_str (self, get_body (self, opts, TRUE));
 }
 
-
-
 const char*
 mu_msg_get_body_text (MuMsg *self, MuMsgOptions opts)
 {
@@ -610,7 +577,6 @@ mu_msg_get_body_text (MuMsg *self, MuMsgOptions opts)
 	return free_later_str (self, get_body (self, opts, FALSE));
 }
 
-
 const GSList*
 mu_msg_get_references (MuMsg *self)
 {
@@ -618,7 +584,6 @@ mu_msg_get_references (MuMsg *self)
 	return get_str_list_field (self, MU_MSG_FIELD_ID_REFS);
 }
 
-
 const GSList*
 mu_msg_get_tags (MuMsg *self)
 {
@@ -626,7 +591,6 @@ mu_msg_get_tags (MuMsg *self)
 	return get_str_list_field (self, MU_MSG_FIELD_ID_TAGS);
 }
 
-
 const char*
 mu_msg_get_field_string (MuMsg *self, MuMsgFieldId mfid)
 {
@@ -634,7 +598,6 @@ mu_msg_get_field_string (MuMsg *self, MuMsgFieldId mfid)
 	return get_str_field (self, mfid);
 }
 
-
 const GSList*
 mu_msg_get_field_string_list (MuMsg *self, MuMsgFieldId mfid)
 {
@@ -642,8 +605,6 @@ mu_msg_get_field_string_list (MuMsg *self, MuMsgFieldId mfid)
 	return get_str_list_field (self, mfid);
 }
 
-
-
 gint64
 mu_msg_get_field_numeric (MuMsg *self, MuMsgFieldId mfid)
 {
@@ -651,7 +612,6 @@ mu_msg_get_field_numeric (MuMsg *self, MuMsgFieldId mfid)
 	return get_num_field (self, mfid);
 }
 
-
 static gboolean
 fill_contact (MuMsgContact *self, InternetAddress *addr,
 	      MuMsgContactType ctype)
@@ -659,30 +619,36 @@ fill_contact (MuMsgContact *self, InternetAddress *addr,
 	if (!addr)
 		return FALSE;
 
+	self->full_address = internet_address_to_string (
+		addr, NULL, FALSE);
+
 	self->name = internet_address_get_name (addr);
+	if (mu_str_is_empty (self->name)) {
+		self->name = NULL;
+	}
+
 	self->type = ctype;
 
 	/* we only support internet mailbox addresses; if we don't
 	 * check, g_mime hits an assert
 	 */
 	if (INTERNET_ADDRESS_IS_MAILBOX(addr))
-		self->address = internet_address_mailbox_get_addr
+		self->email= internet_address_mailbox_get_addr
 			(INTERNET_ADDRESS_MAILBOX(addr));
 	else
-		self->address  = NULL;
+		self->email  = NULL;
 
 	/* if there's no address, just a name, it's probably a local
 	 * address (without @) */
-	if (self->name && !self->address)
-		self->address = self->name;
+	if (self->name && !self->email)
+		self->email = self->name;
 
-	/* note, the address could NULL e.g. when the recipient is something like
-	 * 'Undisclosed recipients'
+	/* note, the address could be NULL e.g. when the recipient is something
+	 * like 'Undisclosed recipients'
 	 */
-	return self->address != NULL;
+	return self->email != NULL;
 }
 
-
 static void
 address_list_foreach (InternetAddressList *addrlist, MuMsgContactType ctype,
 		      MuMsgContactForeachFunc func, gpointer user_data)
@@ -695,18 +661,22 @@ address_list_foreach (InternetAddressList *addrlist, MuMsgContactType ctype,
 	len = internet_address_list_length(addrlist);
 
 	for (i = 0; i != len; ++i) {
-		MuMsgContact contact;
+		MuMsgContact	contact;
+		gboolean	keep_going;
+
 		if (!fill_contact(&contact,
 				  internet_address_list_get_address (addrlist, i),
 				  ctype))
 			continue;
 
-		if (!(func)(&contact, user_data))
+		keep_going = func(&contact, user_data);
+		g_free ((char*)contact.full_address);
+
+		if (!keep_going)
 			break;
 	}
 }
 
-
 static void
 addresses_foreach (const char* addrs, MuMsgContactType ctype,
 		   MuMsgContactForeachFunc func, gpointer user_data)
@@ -716,46 +686,37 @@ addresses_foreach (const char* addrs, MuMsgContactType ctype,
 	if (!addrs)
 		return;
 
-	addrlist = internet_address_list_parse_string (addrs);
+	addrlist = internet_address_list_parse (NULL, addrs);
 	if (addrlist) {
 		address_list_foreach (addrlist, ctype, func, user_data);
 		g_object_unref (addrlist);
 	}
 }
 
-
 static void
 msg_contact_foreach_file (MuMsg *msg, MuMsgContactForeachFunc func,
 			  gpointer user_data)
 {
 	int i;
 	struct {
-		GMimeRecipientType     _gmime_type;
-		MuMsgContactType       _type;
+		GMimeAddressType _gmime_type;
+		MuMsgContactType _type;
 	} ctypes[] = {
-		{GMIME_RECIPIENT_TYPE_TO,  MU_MSG_CONTACT_TYPE_TO},
-		{GMIME_RECIPIENT_TYPE_CC,  MU_MSG_CONTACT_TYPE_CC},
-		{GMIME_RECIPIENT_TYPE_BCC, MU_MSG_CONTACT_TYPE_BCC},
+		{GMIME_ADDRESS_TYPE_FROM,     MU_MSG_CONTACT_TYPE_FROM},
+		{GMIME_ADDRESS_TYPE_REPLY_TO, MU_MSG_CONTACT_TYPE_REPLY_TO},
+		{GMIME_ADDRESS_TYPE_TO,       MU_MSG_CONTACT_TYPE_TO},
+		{GMIME_ADDRESS_TYPE_CC,       MU_MSG_CONTACT_TYPE_CC},
+		{GMIME_ADDRESS_TYPE_BCC,      MU_MSG_CONTACT_TYPE_BCC},
 	};
 
-	/* sender */
-	addresses_foreach (g_mime_message_get_sender (msg->_file->_mime_msg),
-			   MU_MSG_CONTACT_TYPE_FROM, func, user_data);
-
-	/* reply_to */
-	addresses_foreach (g_mime_message_get_reply_to (msg->_file->_mime_msg),
-			   MU_MSG_CONTACT_TYPE_REPLY_TO, func, user_data);
-
-	/* get to, cc, bcc */
 	for (i = 0; i != G_N_ELEMENTS(ctypes); ++i) {
 		InternetAddressList *addrlist;
-		addrlist = g_mime_message_get_recipients (msg->_file->_mime_msg,
-							  ctypes[i]._gmime_type);
+		addrlist = g_mime_message_get_addresses (msg->_file->_mime_msg,
+							 ctypes[i]._gmime_type);
 		address_list_foreach (addrlist, ctypes[i]._type, func, user_data);
 	}
 }
 
-
 static void
 msg_contact_foreach_doc (MuMsg *msg, MuMsgContactForeachFunc func,
 			 gpointer user_data)
@@ -770,7 +731,6 @@ msg_contact_foreach_doc (MuMsg *msg, MuMsgContactForeachFunc func,
 			   MU_MSG_CONTACT_TYPE_BCC, func, user_data);
 }
 
-
 void
 mu_msg_contact_foreach (MuMsg *msg, MuMsgContactForeachFunc func,
 			gpointer user_data)
@@ -786,8 +746,6 @@ mu_msg_contact_foreach (MuMsg *msg, MuMsgContactForeachFunc func,
 		g_return_if_reached ();
 }
 
-
-
 static int
 cmp_str (const char *s1, const char *s2)
 {
@@ -823,7 +781,6 @@ cmp_str (const char *s1, const char *s2)
 	}
 }
 
-
 static int
 cmp_subject (const char* s1, const char *s2)
 {
@@ -839,7 +796,6 @@ cmp_subject (const char* s1, const char *s2)
 		mu_str_subject_normalize (s2));
 }
 
-
 int
 mu_msg_cmp (MuMsg *m1, MuMsg *m2, MuMsgFieldId mfid)
 {
@@ -865,7 +821,6 @@ mu_msg_cmp (MuMsg *m1, MuMsg *m2, MuMsgFieldId mfid)
 	return 0; /* TODO: handle lists */
 }
 
-
 gboolean
 mu_msg_is_readable (MuMsg *self)
 {
@@ -874,8 +829,6 @@ mu_msg_is_readable (MuMsg *self)
 	return access (mu_msg_get_path (self), R_OK) == 0 ? TRUE : FALSE;
 }
 
-
-
 /* we need do to determine the
  *   /home/foo/Maildir/bar
  * from the /bar
@@ -932,7 +885,6 @@ get_target_mdir (MuMsg *msg, const char *target_maildir, GError **err)
 	return rv;
 }
 
-
 /*
  * move a msg to another maildir, trying to maintain 'integrity',
  * ie. msg in 'new/' will go to new/, one in cur/ goes to cur/. be
@@ -973,11 +925,11 @@ mu_msg_move_to_maildir (MuMsg *self, const char *maildir,
 	/* and create a new one */
 	self->_file = mu_msg_file_new (newfullpath, maildir, err);
 	g_free (targetmdir);
+	g_free (newfullpath);
 
 	return self->_file ? TRUE : FALSE;
 }
 
-
 /*
  * Rename a message-file, keeping the same flags. This is useful for tricking
  * some 3rd party progs such as mbsync
@@ -992,3 +944,70 @@ mu_msg_tickle (MuMsg *self, GError **err)
 				       mu_msg_get_flags (self),
 				       FALSE, TRUE, err);
 }
+
+const char*
+mu_str_flags_s  (MuFlags flags)
+{
+	return mu_flags_to_str_s (flags, MU_FLAG_TYPE_ANY);
+}
+
+char*
+mu_str_flags  (MuFlags flags)
+{
+	return g_strdup (mu_str_flags_s(flags));
+}
+
+static void
+cleanup_contact (char *contact)
+{
+	char *c, *c2;
+
+	/* replace "'<> with space */
+	for (c2 = contact; *c2; ++c2)
+		if (*c2 == '"' || *c2 == '\'' || *c2 == '<' || *c2 == '>')
+			*c2 = ' ';
+
+	/* remove everything between '()' if it's after the 5th pos;
+	 * good to cleanup corporate contact address spam... */
+	c = g_strstr_len (contact, -1, "(");
+	if (c && c - contact > 5)
+		*c = '\0';
+
+	g_strstrip (contact);
+}
+
+
+/* this is still somewhat simplistic... */
+const char*
+mu_str_display_contact_s (const char *str)
+{
+	static gchar contact[255];
+	gchar *c, *c2;
+
+	str = str ? str : "";
+	g_strlcpy (contact, str, sizeof(contact));
+
+	/* we check for '<', so we can strip out the address stuff in
+	 * e.g. 'Hello World <hello@world.xx>, but only if there is
+	 * something alphanumeric before the <
+	 */
+	c = g_strstr_len (contact, -1, "<");
+	if (c != NULL) {
+		for (c2 = contact; c2 < c && !(isalnum(*c2)); ++c2);
+		if (c2 != c) /* apparently, there was something,
+			      * so we can remove the <... part*/
+			*c = '\0';
+	}
+
+	cleanup_contact (contact);
+
+	return contact;
+}
+
+char*
+mu_str_display_contact (const char *str)
+{
+	g_return_val_if_fail (str, NULL);
+
+	return g_strdup (mu_str_display_contact_s (str));
+}
