@@ -24,7 +24,7 @@
 #include <mu-flags.h>
 #include <mu-msg-fields.h>
 #include <mu-msg-prio.h>
-#include <mu-util.h> /* for MuError and XapianDocument */
+#include <utils/mu-util.h>
 
 G_BEGIN_DECLS
 
@@ -109,7 +109,7 @@ MuMsg *mu_msg_new_from_doc (XapianDocument* doc, GError **err)
  * @param msg a MuMsg
  * @param err receives error information
  *
- * @return TRUE if this succceeded, FALSE in case of error
+ * @return TRUE if this succeeded, FALSE in case of error
  */
 gboolean mu_msg_load_msg_file (MuMsg *msg, GError **err);
 
@@ -425,7 +425,6 @@ const GSList* mu_msg_get_tags (MuMsg *self);
 int mu_msg_cmp (MuMsg *m1, MuMsg *m2, MuMsgFieldId mfid);
 
 
-
 /**
  * check whether there there's a readable file behind this message
  *
@@ -465,6 +464,30 @@ char* mu_msg_to_sexp (MuMsg *msg, unsigned docid,
 		      MuMsgOptions ops)
 	G_GNUC_MALLOC G_GNUC_WARN_UNUSED_RESULT;
 
+#ifdef HAVE_JSON_GLIB
+
+struct _JsonNode; /* forward declaration */
+
+/**
+ * convert the msg to json
+ *
+ * @param msg a valid message
+ * @param docid the docid for this message, or 0
+ * @param ti thread info for the current message, or NULL
+ * @param opts, bitwise OR'ed;
+ *    - MU_MSG_OPTION_HEADERS_ONLY: only include message fields which can be
+ *      obtained from the database (this is much faster if the MuMsg is
+ *      database-backed, so no file needs to be opened)
+ *    - MU_MSG_OPTION_EXTRACT_IMAGES: extract image attachments as temporary
+ *      files and include links to those in the sexp
+ *
+ * @return a string with the sexp (free with g_free) or NULL in case of error
+ */
+struct _JsonNode* mu_msg_to_json (MuMsg *msg, unsigned docid,
+				  const struct _MuMsgIterThreadInfo *ti,
+				  MuMsgOptions ops) G_GNUC_WARN_UNUSED_RESULT;
+#endif /*HAVE_JSON_GLIB*/
+
 /**
  * move a message to another maildir; note that this does _not_ update
  * the database
@@ -488,7 +511,6 @@ gboolean mu_msg_move_to_maildir (MuMsg *msg, const char *maildir,
 				 gboolean new_name,
 				 GError **err);
 
-
 /**
  * Tickle a message -- ie., rename a message to some new semi-random name,while
  * maintaining the maildir and flags. This can be useful when dealing with
@@ -505,7 +527,6 @@ gboolean mu_msg_move_to_maildir (MuMsg *msg, const char *maildir,
 gboolean mu_msg_tickle (MuMsg *msg, GError **err);
 
 
-
 enum _MuMsgContactType {  /* Reply-To:? */
 	MU_MSG_CONTACT_TYPE_TO    = 0,
 	MU_MSG_CONTACT_TYPE_FROM,
@@ -524,10 +545,11 @@ typedef guint MuMsgContactType;
 	((MCT) < MU_MSG_CONTACT_TYPE_NUM)
 
 struct _MuMsgContact {
-	const char		*name;	    /* Foo Bar */
-	const char		*address;   /* foo@bar.cuux */
-	MuMsgContactType	 type;	    /* MU_MSG_CONTACT_TYPE_{ TO,
-					     * CC, BCC, FROM, REPLY_TO} */
+	const char		*name;	       /**< Foo Bar */
+	const char		*email;        /**< foo@bar.cuux */
+	const char              *full_address; /**< Foo Bar <foo@bar.cuux> */
+	MuMsgContactType	 type;	       /**< MU_MSG_CONTACT_TYPE_{ TO,
+						  CC, BCC, FROM, REPLY_TO} */
 };
 typedef struct _MuMsgContact	 MuMsgContact;
 
@@ -542,13 +564,13 @@ typedef struct _MuMsgContact	 MuMsgContact;
 #define mu_msg_contact_name(ct)    ((ct)->name)
 
 /**
- * macro to get the address of a contact
+ * macro to get the email address of a contact
  *
  * @param ct a MuMsgContact
  *
  * @return the address
  */
-#define mu_msg_contact_address(ct) ((ct)->address)
+#define mu_msg_contact_email(ct) ((ct)->email)
 
 /**
  * macro to get the contact type
@@ -584,6 +606,49 @@ typedef gboolean  (*MuMsgContactForeachFunc) (MuMsgContact* contact,
 void mu_msg_contact_foreach (MuMsg *msg, MuMsgContactForeachFunc func,
 			     gpointer user_data);
 
+
+
+/**
+ * create a 'display contact' from an email header To/Cc/Bcc/From-type address
+ * ie., turn
+ *     "Foo Bar" <foo@bar.com>
+ * into
+ *      Foo Bar
+ * Note that this is based on some simple heuristics. Max length is 255 bytes.
+ *
+ *   mu_str_display_contact_s returns a statically allocated
+ *   buffer (ie, non-reentrant), while mu_str_display_contact
+ *   returns a newly allocated string that you must free with g_free
+ *   when done with it.
+ *
+ * @param str a 'contact str' (ie., what is in the To/Cc/Bcc/From
+ * fields), or NULL
+ *
+ * @return a newly allocated string with a display contact
+ */
+const char* mu_str_display_contact_s (const char *str) G_GNUC_CONST;
+char *mu_str_display_contact (const char *str) G_GNUC_WARN_UNUSED_RESULT;
+
+/**
+ * get a display string for a given set of flags, OR'ed in
+ * @param flags; one character per flag:
+ * D=draft,F=flagged,N=new,P=passed,R=replied,S=seen,T=trashed
+ * a=has-attachment,s=signed, x=encrypted
+ *
+ * mu_str_file_flags_s  returns a ptr to a static buffer,
+ * while mu_str_file_flags returns dynamically allocated
+ * memory that must be freed after use.
+ *
+ * @param flags file flags
+ *
+ * @return a string representation of the flags; see above
+ * for what to do with it
+ */
+const char* mu_str_flags_s  (MuFlags flags) G_GNUC_CONST;
+char*       mu_str_flags    (MuFlags flags)
+    G_GNUC_MALLOC G_GNUC_WARN_UNUSED_RESULT;
+
+
 G_END_DECLS
 
 #endif /*__MU_MSG_H__*/
