@@ -32,15 +32,16 @@
 
 #include "mu-msg-iter.h"
 
-#include "mu-util.h"
-#include "mu-str.h"
-#include "mu-date.h"
+#include "utils/mu-str.h"
+#include "utils/mu-date.h"
+#include <utils/mu-utils.hh>
 
-#include <parser/proc-iface.hh>
-#include <parser/utils.hh>
-#include <parser/xapian.hh>
+#include <query/mu-proc-iface.hh>
+#include <query/mu-xapian.hh>
 
-struct MuProc: public Mux::ProcIface {
+using namespace Mu;
+
+struct MuProc: public Mu::ProcIface {
 
 	MuProc (const Xapian::Database& db): db_{db} {}
 
@@ -146,11 +147,11 @@ struct MuProc: public Mux::ProcIface {
 		std::string	u2 = upper;
 
 		if (id == MU_MSG_FIELD_ID_DATE) {
-			l2 = Mux::date_to_time_t_string (lower, true);
-			u2 = Mux::date_to_time_t_string (upper, false);
+			l2 = Mu::date_to_time_t_string (lower, true);
+			u2 = Mu::date_to_time_t_string (upper, false);
 		} else if (id == MU_MSG_FIELD_ID_SIZE) {
-			l2 = Mux::size_to_string (lower, true);
-			u2 = Mux::size_to_string (upper, false);
+			l2 = Mu::size_to_string (lower, true);
+			u2 = Mu::size_to_string (upper, false);
 		}
 
 		return { l2, u2 };
@@ -186,7 +187,7 @@ public:
 		const auto db = reinterpret_cast<Xapian::Database*>
 			(mu_store_get_read_only_database (_store));
 		if (!db)
-			throw std::runtime_error ("no database");
+			throw Mu::Error(Error::Code::NotFound, "no database");
 		return *db;
 	}
 private:
@@ -196,13 +197,13 @@ private:
 static const Xapian::Query
 get_query (MuQuery *mqx, const char* searchexpr, bool raw, GError **err) try {
 
-	Mux::WarningVec warns;
-	const auto tree = Mux::parse (searchexpr, warns,
+	Mu::WarningVec warns;
+	const auto tree = Mu::parse (searchexpr, warns,
 				      std::make_unique<MuProc>(mqx->db()));
 	for (const auto w: warns)
 		std::cerr << w << std::endl;
 
-	return Mux::xapian_query (tree);
+	return Mu::xapian_query (tree);
 
 } catch (...) {
 	mu_util_g_set_error (err,MU_ERROR_XAPIAN_QUERY,
@@ -215,12 +216,6 @@ mu_query_new (MuStore *store, GError **err)
 {
 	g_return_val_if_fail (store, NULL);
 
-	if (mu_store_count (store, err) == 0) {
-		g_set_error (err, MU_ERROR_DOMAIN, MU_ERROR_XAPIAN_IS_EMPTY,
-			     "database is empty");
-		return 0;
-	}
-
 	try {
 		return new MuQuery (store);
 	} MU_XAPIAN_CATCH_BLOCK_G_ERROR_RETURN (err, MU_ERROR_XAPIAN, 0);
@@ -369,15 +364,23 @@ get_related_query (MuMsgIter *iter, GHashTable **orig_set)
 
 
 static void
-include_related (MuQuery *self, MuMsgIter **iter, int maxnum,
-		 MuMsgFieldId sortfieldid, MuQueryFlags flags)
+get_related_messages (MuQuery *self, MuMsgIter **iter, int maxnum,
+                      MuMsgFieldId sortfieldid, MuQueryFlags flags,
+                      Xapian::Query orig_query)
 {
 	GHashTable *orig_set;
 	Xapian::Enquire enq (self->db());
 	MuMsgIter *rel_iter;
+	const bool inc_related = flags & MU_QUERY_FLAG_INCLUDE_RELATED;
 
 	orig_set = NULL;
-	enq.set_query(get_related_query (*iter, &orig_set));
+	Xapian::Query new_query = get_related_query (*iter, &orig_set);
+	/* If related message are not desired, filter out messages which would not
+	   have matched the original query.
+	 */
+	if (!inc_related)
+	    new_query = Xapian::Query (Xapian::Query::OP_AND, orig_query, new_query);
+	enq.set_query(new_query);
 	enq.set_cutoff(0,0);
 
 	rel_iter= mu_msg_iter_new (
@@ -410,11 +413,12 @@ mu_query_run (MuQuery *self, const char *searchexpr, MuMsgFieldId sortfieldid,
 			      sortfieldid    == MU_MSG_FIELD_ID_NONE,
 			      NULL);
 	try {
-		MuMsgIter	*iter;
-		MuQueryFlags	 first_flags;
-		const auto 	 inc_related  = flags & MU_QUERY_FLAG_INCLUDE_RELATED;
-		const auto	 descending   = flags & MU_QUERY_FLAG_DESCENDING;
-		const auto       raw	      = flags & MU_QUERY_FLAG_RAW;
+		MuMsgIter *iter;
+		MuQueryFlags first_flags;
+		const bool threads     = flags & MU_QUERY_FLAG_THREADS;
+		const bool inc_related = flags & MU_QUERY_FLAG_INCLUDE_RELATED;
+		const bool descending  = flags & MU_QUERY_FLAG_DESCENDING;
+		const bool raw         = flags & MU_QUERY_FLAG_RAW;
 		Xapian::Enquire enq (get_enquire(self, searchexpr, sortfieldid,
 						 descending, raw, err));
 
@@ -426,31 +430,32 @@ mu_query_run (MuQuery *self, const char *searchexpr, MuMsgFieldId sortfieldid,
 
 		/* get the 'real' maxnum if it was specified as < 0 */
 		maxnum = maxnum < 0 ? self->db().get_doccount() : maxnum;
-		/* if we do a include-related query, it's wasted
-		 * effort to calculate threads already in the first
-		 * query since we can do it in the second one
+		/* Calculating threads involves two queries, so do the calculation only in
+		 * the second query instead of in both.
 		 */
-		if (inc_related)
+		if (threads)
 			first_flags = (MuQueryFlags)(flags & ~MU_QUERY_FLAG_THREADS);
 		else
 			first_flags = flags;
-
-		iter   = mu_msg_iter_new (
+		/* Perform the initial query, returning up to max num results.
+		 */
+		iter  = mu_msg_iter_new (
 			reinterpret_cast<XapianEnquire*>(&enq),
 			maxnum,
-			/* with inc_related, we do the sorting in the
-			 * second query
-			 */
-			inc_related ? MU_MSG_FIELD_ID_NONE : sortfieldid,
+			sortfieldid,
 			msg_iter_flags (first_flags),
 			err);
-		/*
-		 * if we want related messages, do a second query,
-		 * based on the message ids / refs of the first one
-		 * */
-		if (inc_related)
-			include_related (self, &iter, maxnum, sortfieldid,
-					 flags);
+		/* If we want threads or related messages, find related messages using a
+		 * second query based on the message ids / refs of the first query's result.
+		 * Do this even if we don't want to include related messages in the final
+		 * result so we can apply the threading algorithm to the related message set
+		 * of a maxnum-sized result instead of the unbounded result of the first
+		 * query. If threads are desired but related message are not, we will remove
+		 * the undesired related messages later.
+		 */
+		if(threads||inc_related)
+			get_related_messages (self, &iter, maxnum, sortfieldid, flags,
+			                      enq.get_query());
 
 		if (err && *err && (*err)->code == MU_ERROR_XAPIAN_MODIFIED) {
 			g_clear_error (err);
@@ -463,6 +468,23 @@ mu_query_run (MuQuery *self, const char *searchexpr, MuMsgFieldId sortfieldid,
 }
 
 
+size_t
+mu_query_count_run (MuQuery *self, const char *searchexpr) try
+{
+	g_return_val_if_fail (self, 0);
+	g_return_val_if_fail (searchexpr, 0);
+
+        const auto enq{get_enquire(self, searchexpr,MU_MSG_FIELD_ID_NONE, false, false, NULL)};
+        auto mset(enq.get_mset(0, self->db().get_doccount()));
+        mset.fetch();
+
+        return mset.size();
+
+} MU_XAPIAN_CATCH_BLOCK_RETURN (0);
+
+
+
+
 char*
 mu_query_internal_xapian (MuQuery *self, const char *searchexpr, GError **err)
 {
@@ -485,8 +507,8 @@ mu_query_internal (MuQuery *self, const char *searchexpr,
 	g_return_val_if_fail (searchexpr, NULL);
 
 	try {
-		Mux::WarningVec warns;
-		const auto tree = Mux::parse (searchexpr, warns,
+		Mu::WarningVec warns;
+		const auto tree = Mu::parse (searchexpr, warns,
 					      std::make_unique<MuProc>(self->db()));
 		std::stringstream ss;
 		ss << tree;
