@@ -1,5 +1,3 @@
-/* -*-mode: c; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-*/
-
 /*
 ** Copyright (C) 2008-2013 Dirk-Jan C. Binnema <djcb@djcbsoftware.nl>
 **
@@ -31,8 +29,6 @@
 #include <signal.h>
 
 #include "mu-msg.h"
-#include "mu-str.h"
-#include "mu-date.h"
 #include "mu-maildir.h"
 #include "mu-index.h"
 #include "mu-query.h"
@@ -40,10 +36,17 @@
 #include "mu-bookmarks.h"
 #include "mu-runtime.h"
 
-#include "mu-util.h"
+#include "utils/mu-util.h"
+#include "utils/mu-str.h"
+#include "utils/mu-date.h"
+
 #include "mu-cmd.h"
 #include "mu-threader.h"
 
+#ifdef HAVE_JSON_GLIB
+#include <json-glib/json-glib.h>
+#endif /*HAVE_JSON_GLIB*/
+
 typedef gboolean (OutputFunc) (MuMsg *msg, MuMsgIter *iter,
 			       MuConfig *opts, GError **err);
 
@@ -67,7 +70,6 @@ print_internal (MuQuery *query, const gchar *expr, gboolean xapian,
 }
 
 
-
 /* returns MU_MSG_FIELD_ID_NONE if there is an error */
 static MuMsgFieldId
 sort_field_from_string (const char* fieldstr, GError **err)
@@ -87,7 +89,6 @@ sort_field_from_string (const char* fieldstr, GError **err)
 	return mfid;
 }
 
-
 static MuMsg*
 get_message (MuMsgIter *iter, time_t after)
 {
@@ -105,7 +106,6 @@ get_message (MuMsgIter *iter, time_t after)
 		return get_message (iter, after);
 	}
 
-
 	if (after != 0 && after > mu_msg_get_timestamp (msg)) {
 		mu_msg_iter_next (iter);
 		return get_message (iter, after);
@@ -124,7 +124,7 @@ run_query (MuQuery *xapian, const gchar *query, MuConfig *opts,  GError **err)
 	sortid = MU_MSG_FIELD_ID_NONE;
 	if (opts->sortfield) {
 		sortid = sort_field_from_string (opts->sortfield, err);
-		if (sortid == MU_MSG_FIELD_ID_NONE) /* error occured? */
+		if (sortid == MU_MSG_FIELD_ID_NONE) /* error occurred? */
 			return FALSE;
 	}
 
@@ -142,7 +142,6 @@ run_query (MuQuery *xapian, const gchar *query, MuConfig *opts,  GError **err)
 	return iter;
 }
 
-
 static gboolean
 exec_cmd (MuMsg *msg, MuMsgIter *iter, MuConfig *opts,  GError **err)
 {
@@ -161,8 +160,6 @@ exec_cmd (MuMsg *msg, MuMsgIter *iter, MuConfig *opts,  GError **err)
 	return rv;
 }
 
-
-
 static gchar*
 resolve_bookmark (MuConfig *opts, GError **err)
 {
@@ -189,7 +186,6 @@ resolve_bookmark (MuConfig *opts, GError **err)
 	return val;
 }
 
-
 static gchar*
 get_query (MuConfig *opts, GError **err)
 {
@@ -222,7 +218,6 @@ get_query (MuConfig *opts, GError **err)
 	return query;
 }
 
-
 static MuQuery*
 get_query_obj (MuStore *store, GError **err)
 {
@@ -235,17 +230,11 @@ get_query_obj (MuStore *store, GError **err)
 		return NULL;
 
 	if (count == 0) {
-		g_set_error (err, MU_ERROR_DOMAIN, MU_ERROR_XAPIAN_IS_EMPTY,
+		g_set_error (err, MU_ERROR_DOMAIN, MU_ERROR_XAPIAN_NEEDS_REINDEX,
 			     "the database is empty");
 		return NULL;
 	}
 
-	if (!mu_store_versions_match (store)) {
-		g_set_error (err, MU_ERROR_DOMAIN, MU_ERROR_XAPIAN_VERSION_MISMATCH,
-			     "the database needs a rebuild");
-		return NULL;
-	}
-
 	mquery = mu_query_new (store, err);
 	if (!mquery)
 		return NULL;
@@ -253,7 +242,6 @@ get_query_obj (MuStore *store, GError **err)
 	return mquery;
 }
 
-
 static gboolean
 prepare_links (MuConfig *opts, GError **err)
 {
@@ -277,15 +265,16 @@ prepare_links (MuConfig *opts, GError **err)
 	return TRUE;
 }
 
-
 static gboolean
 output_link (MuMsg *msg, MuMsgIter *iter, MuConfig *opts,  GError **err)
 {
+	if (mu_msg_iter_is_first (iter) && !prepare_links (opts, err))
+		return FALSE;
+
 	return mu_maildir_link (mu_msg_get_path (msg),
 				opts->linksdir, err);
 }
 
-
 static void
 ansi_color_maybe (MuMsgFieldId mfid, gboolean color)
 {
@@ -320,7 +309,6 @@ ansi_color_maybe (MuMsgFieldId mfid, gboolean color)
 	fputs (ansi, stdout);
 }
 
-
 static void
 ansi_reset_maybe (MuMsgFieldId mfid, gboolean color)
 {
@@ -344,7 +332,8 @@ field_string_list (MuMsg *msg, MuMsgFieldId mfid)
 
 	str = mu_str_from_list (lst, ',');
 	if (str) {
-		strncpy (buf, str, sizeof(buf));
+		strncpy (buf, str, sizeof(buf)-1);
+		buf[sizeof(buf)-1]='\0';
 		g_free (str);
 		return buf;
 	}
@@ -352,8 +341,6 @@ field_string_list (MuMsg *msg, MuMsgFieldId mfid)
 	return NULL;
 }
 
-
-
 static const char*
 display_field (MuMsg *msg, MuMsgFieldId mfid)
 {
@@ -393,7 +380,6 @@ display_field (MuMsg *msg, MuMsgFieldId mfid)
 	}
 }
 
-
 static void
 print_summary (MuMsg *msg, MuConfig *opts)
 {
@@ -416,7 +402,6 @@ print_summary (MuMsg *msg, MuConfig *opts)
 	g_free (summ);
 }
 
-
 static void
 thread_indent (MuMsgIter *iter)
 {
@@ -455,8 +440,6 @@ thread_indent (MuMsgIter *iter)
 	}
 }
 
-
-
 static void
 output_plain_fields (MuMsg *msg, const char *fields,
 		     gboolean color, gboolean threads)
@@ -505,8 +488,6 @@ output_plain (MuMsg *msg, MuMsgIter *iter, MuConfig *opts, GError **err)
 	return TRUE;
 }
 
-
-
 static gboolean
 output_sexp (MuMsg *msg, MuMsgIter *iter, MuConfig *opts, GError **err)
 {
@@ -522,6 +503,41 @@ output_sexp (MuMsg *msg, MuMsgIter *iter, MuConfig *opts, GError **err)
 	return TRUE;
 }
 
+static gboolean
+output_json (MuMsg *msg, MuMsgIter *iter, MuConfig *opts, GError **err)
+{
+#ifdef HAVE_JSON_GLIB
+	JsonNode			*node;
+	const MuMsgIterThreadInfo	*ti;
+	char				*s;
+
+	if (mu_msg_iter_is_first(iter))
+		g_print ("[\n");
+
+	ti   = opts->threads ? mu_msg_iter_get_thread_info (iter) : NULL;
+	node = mu_msg_to_json (msg, mu_msg_iter_get_docid (iter),
+			       ti, MU_MSG_OPTION_HEADERS_ONLY);
+
+	s = json_to_string (node, TRUE);
+	json_node_free (node);
+
+	fputs (s, stdout);
+	g_free (s);
+
+	if (mu_msg_iter_is_last(iter))
+		fputs("]\n", stdout);
+	else
+		fputs (",\n", stdout);
+
+	return TRUE;
+#else
+	g_set_error (err, MU_ERROR_DOMAIN, MU_ERROR_IN_PARAMETERS,
+		     "this mu was built without json support");
+	return FALSE;
+#endif /*HAVE_JSON_GLIB*/
+
+}
+
 static void
 print_attr_xml (const char* elm, const char *str)
 {
@@ -535,10 +551,14 @@ print_attr_xml (const char* elm, const char *str)
 	g_free (esc);
 }
 
-
 static gboolean
 output_xml (MuMsg *msg, MuMsgIter *iter, MuConfig *opts, GError **err)
 {
+	if (mu_msg_iter_is_first(iter)) {
+		g_print ("<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n");
+		g_print ("<messages>\n");
+	}
+
 	g_print ("\t<message>\n");
 	print_attr_xml ("from", mu_msg_get_from (msg));
 	print_attr_xml ("to", mu_msg_get_to (msg));
@@ -552,29 +572,22 @@ output_xml (MuMsg *msg, MuMsgIter *iter, MuConfig *opts, GError **err)
 	print_attr_xml ("maildir", mu_msg_get_maildir (msg));
 	g_print ("\t</message>\n");
 
+	if (mu_msg_iter_is_last(iter))
+		g_print ("</messages>\n");
+
 	return TRUE;
 }
 
-
 static OutputFunc*
-output_prepare (MuConfig *opts, GError **err)
+get_output_func (MuConfig *opts, GError **err)
 {
 	switch (opts->format) {
-	case MU_CONFIG_FORMAT_EXEC:
-		return exec_cmd;
-	case MU_CONFIG_FORMAT_LINKS:
-		if (!prepare_links (opts, err))
-			return NULL;
-		else
-			return output_link;
-	case MU_CONFIG_FORMAT_PLAIN:
-		return output_plain;
-	case MU_CONFIG_FORMAT_XML:
-		g_print ("<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n");
-		g_print ("<messages>\n");
-		return output_xml;
-	case MU_CONFIG_FORMAT_SEXP:
-		return output_sexp;
+	case MU_CONFIG_FORMAT_LINKS: return output_link;
+	case MU_CONFIG_FORMAT_EXEC:  return exec_cmd;
+	case MU_CONFIG_FORMAT_PLAIN: return output_plain;
+	case MU_CONFIG_FORMAT_XML:   return output_xml;
+	case MU_CONFIG_FORMAT_SEXP:  return output_sexp;
+	case MU_CONFIG_FORMAT_JSON:  return output_json;
 
 	default:
 		g_return_val_if_reached (NULL);
@@ -582,14 +595,6 @@ output_prepare (MuConfig *opts, GError **err)
 	}
 }
 
-static void
-output_finish (MuConfig *opts)
-{
-	if (opts->format == MU_CONFIG_FORMAT_XML)
-		g_print ("</messages>\n");
-}
-
-
 static gboolean
 output_query_results (MuMsgIter *iter, MuConfig *opts, GError **err)
 {
@@ -597,7 +602,7 @@ output_query_results (MuMsgIter *iter, MuConfig *opts, GError **err)
 	gboolean	 rv;
 	OutputFunc	*output_func;
 
-	output_func = output_prepare (opts, err);
+	output_func = get_output_func (opts, err);
 	if (!output_func)
 		return FALSE;
 
@@ -624,8 +629,6 @@ output_query_results (MuMsgIter *iter, MuConfig *opts, GError **err)
 			++count;
 	}
 
-	output_finish (opts);
-
 	if (rv && count == 0) {
 		mu_util_g_set_error (err, MU_ERROR_NO_MATCHES,
 				     "no matches for search expression");
@@ -635,7 +638,6 @@ output_query_results (MuMsgIter *iter, MuConfig *opts, GError **err)
 	return rv;
 }
 
-
 static gboolean
 process_query (MuQuery *xapian, const gchar *query, MuConfig *opts, GError **err)
 {
@@ -652,7 +654,6 @@ process_query (MuQuery *xapian, const gchar *query, MuConfig *opts, GError **err
 	return rv;
 }
 
-
 static gboolean
 execute_find (MuStore *store, MuConfig *opts, GError **err)
 {
@@ -684,7 +685,6 @@ execute_find (MuStore *store, MuConfig *opts, GError **err)
 	return rv;
 }
 
-
 static gboolean
 format_params_valid (MuConfig *opts, GError **err)
 {
@@ -693,6 +693,7 @@ format_params_valid (MuConfig *opts, GError **err)
 		break;
 	case MU_CONFIG_FORMAT_PLAIN:
 	case MU_CONFIG_FORMAT_SEXP:
+	case MU_CONFIG_FORMAT_JSON:
 	case MU_CONFIG_FORMAT_LINKS:
 	case MU_CONFIG_FORMAT_XML:
 	case MU_CONFIG_FORMAT_XQUERY:
@@ -746,7 +747,6 @@ query_params_valid (MuConfig *opts, GError **err)
 	return FALSE;
 }
 
-
 MuError
 mu_cmd_find (MuStore *store, MuConfig *opts, GError **err)
 {
