@@ -1,7 +1,7 @@
 /* -*-mode: c; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-*/
 
 /*
-** Copyright (C) 2008-2013 Dirk-Jan C. Binnema <djcb@djcbsoftware.nl>
+** Copyright (C) 2008-2016 Dirk-Jan C. Binnema <djcb@djcbsoftware.nl>
 **
 ** This program is free software; you can redistribute it and/or modify it
 ** under the terms of the GNU General Public License as published by the
@@ -31,12 +31,13 @@
 #include <signal.h>
 #include <unistd.h>
 
-#include "mu-util.h"
 #include "mu-msg.h"
 #include "mu-index.h"
-#include "mu-store.h"
+#include "mu-store.hh"
 #include "mu-runtime.h"
-#include "mu-log.h"
+
+#include "utils/mu-util.h"
+#include "utils/mu-log.h"
 
 static gboolean MU_CAUGHT_SIGNAL;
 
@@ -49,24 +50,24 @@ sig_handler (int sig)
 			   "press again to kill immediately");
 	}
 
-        MU_CAUGHT_SIGNAL = TRUE;
+	MU_CAUGHT_SIGNAL = TRUE;
 }
 
 static void
 install_sig_handler (void)
 {
-        struct sigaction action;
-        int i, sigs[] = { SIGINT, SIGHUP, SIGTERM };
+	struct sigaction action;
+	int i, sigs[] = { SIGINT, SIGHUP, SIGTERM };
 
-        MU_CAUGHT_SIGNAL = FALSE;
+	MU_CAUGHT_SIGNAL = FALSE;
 
-        action.sa_handler = sig_handler;
-        sigemptyset(&action.sa_mask);
-        action.sa_flags = SA_RESETHAND;
+	action.sa_handler = sig_handler;
+	sigemptyset(&action.sa_mask);
+	action.sa_flags = SA_RESETHAND;
 
-        for (i = 0; i != G_N_ELEMENTS(sigs); ++i)
-                if (sigaction (sigs[i], &action, NULL) != 0)
-                        g_critical ("set sigaction for %d failed: %s",
+	for (i = 0; i != G_N_ELEMENTS(sigs); ++i)
+		if (sigaction (sigs[i], &action, NULL) != 0)
+			g_critical ("set sigaction for %d failed: %s",
 				    sigs[i], strerror (errno));;
 }
 
@@ -81,12 +82,6 @@ check_params (MuConfig *opts, GError **err)
 		return FALSE;
 	}
 
-	if (opts->xbatchsize < 0) {
-		mu_util_g_set_error (err, MU_ERROR_IN_PARAMETERS,
-				     "the batch size must be >= 0");
-		return FALSE;
-	}
-
 	if (opts->max_msg_size < 0) {
 		mu_util_g_set_error (err, MU_ERROR_IN_PARAMETERS,
 				     "the maximum message size must >= 0");
@@ -96,32 +91,6 @@ check_params (MuConfig *opts, GError **err)
 	return TRUE;
 }
 
-static gboolean
-check_maildir (const char *maildir, GError **err)
-{
-	if (!maildir) {
-		mu_util_g_set_error (err, MU_ERROR_IN_PARAMETERS,
-				     "no maildir to work on; use --maildir=");
-		return FALSE;
-	}
-
-	if (!g_path_is_absolute (maildir)) {
-		mu_util_g_set_error (err, MU_ERROR_IN_PARAMETERS,
-				     "maildir path '%s' is not absolute",
-				     maildir);
-		return FALSE;
-	}
-
-	if (!mu_util_check_dir (maildir, TRUE, FALSE)) {
-		mu_util_g_set_error (err, MU_ERROR_IN_PARAMETERS,
-				     "not a valid Maildir: %s", maildir);
-		return FALSE;
-	}
-
-	return TRUE;
-}
-
-
 static MuError
 index_msg_silent_cb (MuIndexStats* stats, void *user_data)
 {
@@ -142,7 +111,7 @@ print_stats (MuIndexStats* stats, gboolean clear, gboolean color)
 		fputs ("\r", stdout);
 
 	if (color)
-		snprintf
+		g_snprintf
 			(output, sizeof(output),
 			 MU_COLOR_YELLOW "%c " MU_COLOR_DEFAULT
 			 "processing mail; "
@@ -154,7 +123,7 @@ print_stats (MuIndexStats* stats, gboolean clear, gboolean color)
 			 (unsigned)stats->_updated,
 			 (unsigned)stats->_cleaned_up);
 	else
-		snprintf
+		g_snprintf
 			(output, sizeof(output),
 			 "%c processing mail; processed: %u; "
 			 "updated/new: %u, cleaned-up: %u",
@@ -185,36 +154,6 @@ index_msg_cb  (MuIndexStats* stats, IndexData *idata)
 	return MU_CAUGHT_SIGNAL ? MU_STOP: MU_OK;
 }
 
-
-
-static gboolean
-database_version_check_and_update (MuStore *store, MuConfig *opts,
-				   GError **err)
-{
-	if (mu_store_count (store, err) == 0)
-		return TRUE;
-
-	/* when rebuilding, we empty the database before doing
-	 * anything */
-	if (opts->rebuild) {
-		g_debug ("clearing database");
-		g_debug ("clearing contacts-cache");
-		return mu_store_clear (store, err);
-	}
-
-	if (mu_store_versions_match (store))
-		return TRUE; /* ok, nothing to do */
-
-	/* ok, database is not up to date */
-	if (opts->autoupgrade) {
-		g_debug ("auto-upgrade: clearing old database and cache");
-		return mu_store_clear (store, err);
-	}
-
-	return FALSE;
-}
-
-
 static void
 show_time (unsigned t, unsigned processed, gboolean color)
 {
@@ -286,22 +225,6 @@ cleanup_missing (MuIndex *midx, MuConfig *opts, MuIndexStats *stats,
 	return (rv == MU_OK || rv == MU_STOP) ? MU_OK: MU_G_ERROR_CODE(err);
 }
 
-
-static void
-index_title (const char* maildir, const char* xapiandir, gboolean color)
-{
-	if (color)
-		g_print ("indexing messages under "
-			   MU_COLOR_BLUE "%s" MU_COLOR_DEFAULT
-			   " ["
-			   MU_COLOR_BLUE "%s" MU_COLOR_DEFAULT
-			   "]\n", maildir, xapiandir);
-	else
-		g_print ("indexing messages under %s [%s]\n",
-			 maildir, xapiandir);
-}
-
-
 static MuError
 cmd_index (MuIndex *midx, MuConfig *opts, MuIndexStats *stats, GError **err)
 {
@@ -309,17 +232,14 @@ cmd_index (MuIndex *midx, MuConfig *opts, MuIndexStats *stats, GError **err)
 	MuError		rv;
 	gboolean	show_progress;
 
-	if (!opts->quiet)
-		index_title (opts->maildir,
-			     mu_runtime_path(MU_RUNTIME_PATH_XAPIANDB),
-			     !opts->nocolor);
-
 	show_progress = !opts->quiet && isatty(fileno(stdout));
 	idata.color   = !opts->nocolor;
 
 	newline_before_on();
 
-	rv = mu_index_run (midx, opts->maildir, opts->rebuild, stats,
+	rv = mu_index_run (midx,
+			   opts->rebuild,
+			   opts->lazycheck, stats,
 			   show_progress ?
 			   (MuIndexMsgCallback)index_msg_cb :
 			   (MuIndexMsgCallback)index_msg_silent_cb,
@@ -344,31 +264,22 @@ init_mu_index (MuStore *store, MuConfig *opts, GError **err)
 	if (!check_params (opts, err))
 		return NULL;
 
-	if (!database_version_check_and_update(store, opts, err))
-		return NULL;
-
-
-	if (!check_maildir (opts->maildir, err))
-		return NULL;
-
 	midx = mu_index_new (store, err);
 	if (!midx)
 		return NULL;
 
 	mu_index_set_max_msg_size (midx, opts->max_msg_size);
-	mu_index_set_xbatch_size (midx, opts->xbatchsize);
 
 	return midx;
 }
 
-
 MuError
 mu_cmd_index (MuStore *store, MuConfig *opts, GError **err)
 {
-	MuIndex *midx;
-	MuIndexStats stats;
-	gboolean rv;
-	time_t t;
+	MuIndex		*midx;
+	MuIndexStats	 stats;
+	gboolean	 rv;
+	time_t		 t;
 
 	g_return_val_if_fail (opts, FALSE);
 	g_return_val_if_fail (opts->cmd == MU_CONFIG_CMD_INDEX,
@@ -382,6 +293,9 @@ mu_cmd_index (MuStore *store, MuConfig *opts, GError **err)
 	mu_index_stats_clear (&stats);
 	install_sig_handler ();
 
+	if (!opts->quiet)
+		mu_store_print_info (store, opts->nocolor);
+
 	t = time (NULL);
 	rv = cmd_index (midx, opts, &stats, err);
 
