@@ -1,6 +1,5 @@
-/* -*-mode: c; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-*/
 /*
-** Copyright (C) 2010-2016 Dirk-Jan C. Binnema <djcb@djcbsoftware.nl>
+** Copyright (C) 2010-2020 Dirk-Jan C. Binnema <djcb@djcbsoftware.nl>
 **
 ** This program is free software; you can redistribute it and/or modify it
 ** under the terms of the GNU General Public License as published by the
@@ -18,9 +17,7 @@
 **
 */
 
-#if HAVE_CONFIG_H
 #include "config.h"
-#endif /*HAVE_CONFIG_H*/
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -31,15 +28,15 @@
 #include "mu-msg.h"
 #include "mu-msg-part.h"
 #include "mu-cmd.h"
-#include "mu-util.h"
-#include "mu-str.h"
-#include "mu-date.h"
 #include "mu-maildir.h"
-#include "mu-contacts.h"
+#include "mu-contacts.hh"
 #include "mu-runtime.h"
 #include "mu-flags.h"
-#include "mu-store.h"
-#include "mu-log.h"
+
+#include "utils/mu-log.h"
+#include "utils/mu-util.h"
+#include "utils/mu-str.h"
+#include "utils/mu-date.h"
 
 #define VIEW_TERMINATOR '\f' /* form-feed */
 
@@ -198,7 +195,6 @@ handle_msg (const char *fname, MuConfig *opts, GError **err)
 	MuMsg *msg;
 	gboolean rv;
 
-	err = NULL;
 	msg = mu_msg_new_from_file (fname, NULL, err);
 	if (!msg)
 		return FALSE;
@@ -244,8 +240,8 @@ view_params_valid (MuConfig *opts, GError **err)
 }
 
 
-MuError
-mu_cmd_view (MuConfig *opts, GError **err)
+static MuError
+cmd_view (MuConfig *opts, GError **err)
 {
 	int i;
 	gboolean rv;
@@ -276,9 +272,8 @@ leave:
 	return MU_OK;
 }
 
-
-MuError
-mu_cmd_mkdir (MuConfig *opts, GError **err)
+static MuError
+cmd_mkdir (MuConfig *opts, GError **err)
 {
 	int i;
 
@@ -319,41 +314,52 @@ check_file_okay (const char *path, gboolean cmd_add)
 }
 
 
-MuError
-mu_cmd_add (MuStore *store, MuConfig *opts, GError **err)
-{
-	gboolean allok;
-	int i;
+typedef gboolean (*ForeachMsgFunc) (MuStore *store, const char *path,
+				    GError **err);
 
-	g_return_val_if_fail (store, MU_ERROR_INTERNAL);
-	g_return_val_if_fail (opts, MU_ERROR_INTERNAL);
-	g_return_val_if_fail (opts->cmd == MU_CONFIG_CMD_ADD,
-			      MU_ERROR_INTERNAL);
+
+static MuError
+foreach_msg_file (MuStore *store, MuConfig *opts,
+		  ForeachMsgFunc foreach_func, GError **err)
+{
+	unsigned	u;
+	gboolean	all_ok;
 
 	/* note: params[0] will be 'add' */
 	if (!opts->params[0] || !opts->params[1]) {
-		g_print ("usage: mu add <file> [<files>]\n");
+		g_print ("usage: mu %s <file> [<files>]\n",
+			 opts->params[0] ? opts->params[0] : "<cmd>");
 		mu_util_g_set_error (err, MU_ERROR_IN_PARAMETERS,
-				     "missing source and/or target");
+				     "missing parameters");
 		return MU_ERROR_IN_PARAMETERS;
 	}
 
-	for (i = 1, allok = TRUE; opts->params[i]; ++i) {
+	for (u = 1, all_ok = TRUE; opts->params[u]; ++u) {
 
-		const char* src;
-		src = opts->params[i];
+		const char* path;
 
-		if (!check_file_okay (src, TRUE) ||
-		    mu_store_add_path (store, src, NULL, err) ==
-		    MU_STORE_INVALID_DOCID) {
-			MU_WRITE_LOG ("failed to add %s", src);
-			allok = FALSE;
+		path = opts->params[u];
+
+		if (!check_file_okay (path, TRUE)) {
+			all_ok = FALSE;
+			MU_WRITE_LOG ("not a valid message file: %s", path);
+			continue;
+		}
+
+		if (!foreach_func (store, path, err)) {
+			all_ok = FALSE;
+			MU_WRITE_LOG ("error with %s: %s", path,
+				      (err&&*err) ? (*err)->message :
+				      "something went wrong");
+			g_clear_error (err);
+			continue;
 		}
 	}
 
-	if (!allok) {
+	if (!all_ok) {
 		mu_util_g_set_error (err, MU_ERROR_XAPIAN_STORE_FAILED,
-				     "store failed for some message(s)");
+				     "%s failed for some message(s)",
+				     opts->params[0]);
 		return MU_ERROR_XAPIAN_STORE_FAILED;
 	}
 
@@ -361,46 +367,72 @@ mu_cmd_add (MuStore *store, MuConfig *opts, GError **err)
 }
 
 
-MuError
-mu_cmd_remove (MuStore *store, MuConfig *opts, GError **err)
+static gboolean
+add_path_func (MuStore *store, const char *path, GError **err)
 {
-	gboolean allok;
-	int i;
+	return mu_store_add_path (store, path, err);
+}
 
+
+static MuError
+cmd_add (MuStore *store, MuConfig *opts, GError **err)
+{
+	g_return_val_if_fail (store, MU_ERROR_INTERNAL);
+	g_return_val_if_fail (opts, MU_ERROR_INTERNAL);
+	g_return_val_if_fail (opts->cmd == MU_CONFIG_CMD_ADD,
+			      MU_ERROR_INTERNAL);
+
+	return foreach_msg_file (store, opts, add_path_func, err);
+}
+
+static gboolean
+remove_path_func (MuStore *store, const char *path, GError **err)
+{
+	if (!mu_store_remove_path (store, path)) {
+		mu_util_g_set_error (err, MU_ERROR_XAPIAN_REMOVE_FAILED,
+				     "failed to remove %s", path);
+		return FALSE;
+	}
+
+	return TRUE;
+}
+
+static MuError
+cmd_remove (MuStore *store, MuConfig *opts, GError **err)
+{
 	g_return_val_if_fail (opts, MU_ERROR_INTERNAL);
 	g_return_val_if_fail (opts->cmd == MU_CONFIG_CMD_REMOVE,
 			      MU_ERROR_INTERNAL);
 
-	/* note: params[0] will be 'remove' */
-	if (!opts->params[0] || !opts->params[1]) {
-		g_warning ("usage: mu remove <file> [<files>]");
-		mu_util_g_set_error (err, MU_ERROR_IN_PARAMETERS,
-				     "missing source and/or target");
-		return MU_ERROR_IN_PARAMETERS;
-	}
+	return foreach_msg_file (store, opts, remove_path_func, err);
+}
 
-	for (i = 1, allok = TRUE; opts->params[i]; ++i) {
+static gboolean
+tickle_func (MuStore *store, const char *path, GError **err)
+{
+	MuMsg		*msg;
+	gboolean	 rv;
 
-		const char* src;
-		src = opts->params[i];
+	msg = mu_msg_new_from_file (path, NULL, err);
+	if (!msg)
+		return FALSE;
 
-		if (!check_file_okay (src, FALSE) ||
-		    !mu_store_remove_path (store, src)) {
-			allok = FALSE;
-			MU_WRITE_LOG ("failed to remove %s", src);
-		}
-	}
+	rv = mu_msg_tickle (msg, err);
+	mu_msg_unref (msg);
 
-	if (!allok) {
-		g_set_error (err, MU_ERROR_DOMAIN, MU_ERROR_XAPIAN_STORE_FAILED,
-			     "remove failed for some message(s)");
-		return MU_ERROR_XAPIAN_REMOVE_FAILED;
-	}
-
-	return MU_OK;
+	return rv;
 }
 
 
+static MuError
+cmd_tickle (MuStore *store, MuConfig *opts, GError **err)
+{
+	g_return_val_if_fail (opts, MU_ERROR_INTERNAL);
+	g_return_val_if_fail (opts->cmd == MU_CONFIG_CMD_TICKLE,
+			      MU_ERROR_INTERNAL);
+
+	return foreach_msg_file (store, opts, tickle_func, err);
+}
 
 struct _VData {
 	MuMsgPartSigStatus combined_status;
@@ -477,8 +509,8 @@ print_verdict (VData *vdata, gboolean color, gboolean verbose)
 }
 
 
-MuError
-mu_cmd_verify (MuConfig *opts, GError **err)
+static MuError
+cmd_verify (MuConfig *opts, GError **err)
 {
 	MuMsg *msg;
 	MuMsgOptions msgopts;
@@ -519,6 +551,49 @@ mu_cmd_verify (MuConfig *opts, GError **err)
 		MU_OK : MU_ERROR;
 }
 
+static MuError
+cmd_info (MuStore *store, MuConfig *opts, GError **err)
+{
+	mu_store_print_info (store, opts->nocolor);
+
+	return MU_OK;
+}
+
+
+static MuError
+cmd_init (MuConfig *opts, GError **err)
+{
+	MuStore	   *store;
+	const char *path;
+
+        /* not provided, nor could we find a good default */
+        if (!opts->maildir) {
+                mu_util_g_set_error (err, MU_ERROR_IN_PARAMETERS,
+				     "missing --maildir parameter and could "
+                                     "not determine default");
+		return MU_ERROR_IN_PARAMETERS;
+        }
+
+	path  = mu_runtime_path(MU_RUNTIME_PATH_XAPIANDB);
+	store = mu_store_new_create (path,
+				     opts->maildir,
+				     (const char**)opts->my_addresses,
+				     err);
+	if (!store)
+		return MU_G_ERROR_CODE(err);
+
+	if (!opts->quiet) {
+		mu_store_print_info (store, opts->nocolor);
+		g_print ("\nstore created.\n"
+			 "use 'mu index' to fill the database "
+                         "with your messages.\n"
+			 "see mu-index(1) for details\n");
+	}
+
+	mu_store_unref (store);
+	return MU_OK;
+}
+
 
 static void
 show_usage (void)
@@ -532,31 +607,41 @@ show_usage (void)
 
 typedef MuError (*store_func) (MuStore *, MuConfig *, GError **err);
 
+
 static MuError
-with_store (store_func func, MuConfig *opts, gboolean read_only,
-	    GError **err)
+with_store (store_func func, MuConfig *opts, gboolean read_only, GError **err)
 {
-	MuStore *store;
-	MuError merr;
+	MuError		 merr;
+	MuStore		*store;
+	const char	*path;
+
+	path  = mu_runtime_path(MU_RUNTIME_PATH_XAPIANDB);
 
 	if (read_only)
-		store = mu_store_new_read_only
-			(mu_runtime_path(MU_RUNTIME_PATH_XAPIANDB),
-			 err);
+		store = mu_store_new_readable (path, err);
 	else
-		store = mu_store_new_writable
-			(mu_runtime_path(MU_RUNTIME_PATH_XAPIANDB),
-			 mu_runtime_path(MU_RUNTIME_PATH_CONTACTS),
-			 opts->rebuild, err);
+		store = mu_store_new_writable (path, err);
+
 	if (!store)
 		return MU_G_ERROR_CODE(err);
 
-	mu_store_set_my_addresses (store, (const char**)opts->my_addresses);
 	merr = func (store, opts, err);
 	mu_store_unref (store);
+
 	return merr;
 }
 
+static MuError
+with_readonly_store (store_func func, MuConfig *opts, GError **err)
+{
+	return with_store (func, opts, TRUE, err);
+}
+
+static MuError
+with_writable_store (store_func func, MuConfig *opts, GError **err)
+{
+	return with_store (func, opts, FALSE, err);
+}
 
 static gboolean
 check_params (MuConfig *opts, GError **err)
@@ -571,7 +656,6 @@ check_params (MuConfig *opts, GError **err)
 	return TRUE;
 }
 
-
 static void
 set_log_options (MuConfig *opts)
 {
@@ -589,8 +673,6 @@ set_log_options (MuConfig *opts)
 		logopts |= MU_LOG_OPTIONS_DEBUG;
 }
 
-
-
 MuError
 mu_cmd_execute (MuConfig *opts, GError **err)
 {
@@ -604,26 +686,44 @@ mu_cmd_execute (MuConfig *opts, GError **err)
 	set_log_options (opts);
 
 	switch (opts->cmd) {
+
 		/* already handled in mu-config.c */
 	case MU_CONFIG_CMD_HELP: return MU_OK;
 
-	case MU_CONFIG_CMD_CFIND:   merr = mu_cmd_cfind   (opts, err); break;
-	case MU_CONFIG_CMD_MKDIR:   merr = mu_cmd_mkdir   (opts, err); break;
+        /* no store needed */
+
+	case MU_CONFIG_CMD_MKDIR:   merr = cmd_mkdir   (opts, err); break;
 	case MU_CONFIG_CMD_SCRIPT:  merr = mu_cmd_script  (opts, err); break;
-	case MU_CONFIG_CMD_VIEW:    merr = mu_cmd_view    (opts, err); break;
-	case MU_CONFIG_CMD_VERIFY:  merr = mu_cmd_verify  (opts, err); break;
+	case MU_CONFIG_CMD_VIEW:    merr = cmd_view    (opts, err); break;
+	case MU_CONFIG_CMD_VERIFY:  merr = cmd_verify  (opts, err); break;
 	case MU_CONFIG_CMD_EXTRACT: merr = mu_cmd_extract (opts, err); break;
 
+	/* read-only store */
+
+	case MU_CONFIG_CMD_CFIND:
+		merr = with_readonly_store (mu_cmd_cfind, opts, err); break;
 	case MU_CONFIG_CMD_FIND:
-		merr = with_store (mu_cmd_find, opts, TRUE, err);      break;
-	case MU_CONFIG_CMD_INDEX:
-		merr = with_store (mu_cmd_index, opts, FALSE, err);    break;
+		merr = with_readonly_store (mu_cmd_find, opts, err);  break;
+	case MU_CONFIG_CMD_INFO:
+		merr = with_readonly_store (cmd_info, opts, err);  break;
+
+	/* writable store */
+
 	case MU_CONFIG_CMD_ADD:
-		merr = with_store (mu_cmd_add, opts, FALSE, err);      break;
+		merr = with_writable_store (cmd_add, opts, err);      break;
 	case MU_CONFIG_CMD_REMOVE:
-		merr = with_store (mu_cmd_remove, opts, FALSE, err);   break;
+		merr = with_writable_store (cmd_remove, opts, err);   break;
+	case MU_CONFIG_CMD_TICKLE:
+		merr = with_writable_store (cmd_tickle, opts, err);   break;
+	case MU_CONFIG_CMD_INDEX:
+		merr = with_writable_store (mu_cmd_index, opts, err);   break;
+
+	/* commands instantiate store themselves */
+	case MU_CONFIG_CMD_INIT:
+		merr = cmd_init (opts,err); break;
 	case MU_CONFIG_CMD_SERVER:
-		merr = with_store (mu_cmd_server, opts, FALSE, err);   break;
+		merr = mu_cmd_server (opts, err); break;
+
 	default:
 		merr = MU_ERROR_IN_PARAMETERS; break;
 	}
