@@ -1,7 +1,5 @@
-/* -*-mode: c; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-*/
-
 /*
-** Copyright (C) 2008-2013 Dirk-Jan C. Binnema <djcb@djcbsoftware.nl>
+** Copyright (C) 2008-2020 Dirk-Jan C. Binnema <djcb@djcbsoftware.nl>
 **
 ** This program is free software; you can redistribute it and/or modify it
 ** under the terms of the GNU General Public License as published by the
@@ -19,53 +17,15 @@
 **
 */
 
-
 #ifndef __MU_CMD_H__
 #define __MU_CMD_H__
 
 #include <glib.h>
 #include <mu-config.h>
-#include <mu-store.h>
+#include <mu-store.hh>
 
 G_BEGIN_DECLS
 
-/**
- * execute the 'mkdir' command
- *
- * @param opts configuration options
- * @param err receives error information, or NULL
- *
- * @return MU_OK (0) if the command succeeded,
- * some error code otherwise
- */
-MuError mu_cmd_mkdir (MuConfig *opts, GError **err);
-
-
-/**
- * execute the 'view' command
- *
- * @param opts configuration options
- * @param err receives error information, or NULL
- *
- * @return MU_OK (0) if the command succeeded,
- * some error code otherwise
- */
-MuError mu_cmd_view (MuConfig *opts, GError **err);
-
-
-/**
- * execute the 'index' command
- *
- * @param store store object to use
- * @param opts configuration options
- * @param err receives error information, or NULL
- *
- * @return MU_OK (0) if the command succeeded,
- * some error code otherwise
- */
-MuError mu_cmd_index   (MuStore *store, MuConfig *opt, GError **err);
-
-
 /**
  * execute the 'find' command
  *
@@ -92,18 +52,6 @@ MuError mu_cmd_find (MuStore *store, MuConfig *opts, GError **err);
 MuError mu_cmd_extract (MuConfig *opts, GError **err);
 
 
-/**
- * execute the 'mv' command
- *
- * @param opts configuration options
- * @param err receives error information, or NULL
- *
- * @return MU_OK (0) if the command succeeds,
- * some error code otherwise
- */
-MuError mu_cmd_mv (MuConfig *opts, GError **err);
-
-
 /**
  * execute the 'script' command
  *
@@ -118,18 +66,6 @@ MuError mu_cmd_script (MuConfig *opts, GError **err);
 /**
  * execute the cfind command
  *
- * @param opts configuration options
- * @param err receives error information, or NULL
- *
- * @return MU_OK (0) if the command succeeds,
- * some error code otherwise
- */
-MuError mu_cmd_cfind (MuConfig *opts, GError **err);
-
-
-/**
- * execute the add command
- *
  * @param store store object to use
  * @param opts configuration options
  * @param err receives error information, or NULL
@@ -137,42 +73,7 @@ MuError mu_cmd_cfind (MuConfig *opts, GError **err);
  * @return MU_OK (0) if the command succeeds,
  * some error code otherwise
  */
-MuError mu_cmd_add (MuStore *store, MuConfig *opts, GError **err);
-
-/**
- * execute the remove command
- *
- * @param store store object to use
- * @param opts configuration options
- * @param err receives error information, or NULL
- *
- * @return MU_OK (0) if the command succeeds,
- * some error code otherwise
- */
-MuError mu_cmd_remove (MuStore *store, MuConfig *opts, GError **err);
-
-
-/**
- * execute the server command
- * @param store store object to use
- * @param opts configuration options
-  *
- * @return MU_OK (0) if the command succeeds,
- * some error code otherwise
- */
-MuError mu_cmd_server (MuStore *store, MuConfig *opts, GError**/*unused*/);
-
-/**
- * execute the verify command (to verify signatures)
- * @param store store object to use
- * @param opts configuration options
- * @param err receives error information, or NULL
- *
- * @return MU_OK (0) if the command succeeds,
- * some error code otherwise
- */
-MuError mu_cmd_verify (MuConfig *opts, GError **err);
-
+MuError mu_cmd_cfind (MuStore *store, MuConfig *opts, GError **err);
 
 /**
  * execute some mu command, based on 'opts'
@@ -184,6 +85,27 @@ MuError mu_cmd_verify (MuConfig *opts, GError **err);
  */
 MuError mu_cmd_execute (MuConfig *opts, GError **err);
 
+/**
+ * execute the 'index' command
+ *
+ * @param store store object to use
+ * @param opts configuration options
+ * @param err receives error information, or NULL
+ *
+ * @return MU_OK (0) if the command succeeded,
+ * some error code otherwise
+ */
+MuError mu_cmd_index (MuStore *store, MuConfig *opt, GError **err);
+
+/**
+ * execute the server command
+ * @param opts configuration options
+ * @param err receives error information, or NULL
+ *
+ * @return MU_OK (0) if the command succeeds, some error code otherwise
+ */
+MuError mu_cmd_server (MuConfig *opts, GError **err);
+
 G_END_DECLS
 
 #endif /*__MU_CMD_H__*/
