@@ -1,7 +1,5 @@
-/* -*-Mode: c; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-*/
-
 /*
-** Copyright (C) 2008-2013 Dirk-Jan C. Binnema <djcb@djcbsoftware.nl>
+** Copyright (C) 2008-2020 Dirk-Jan C. Binnema <djcb@djcbsoftware.nl>
 **
 ** This program is free software; you can redistribute it and/or modify it
 ** under the terms of the GNU General Public License as published by the
@@ -28,12 +26,15 @@
 #include <unistd.h>
 #include <stdio.h>
 
-#include "mu-util.h"
 #include "mu-config.h"
 #include "mu-cmd.h"
 
+
 static MuConfig MU_CONFIG;
 
+#define color_maybe(C) (MU_CONFIG.nocolor ? "" : (C))
+
+
 static MuConfigFormat
 get_output_format (const char *formatstr)
 {
@@ -51,9 +52,11 @@ get_output_format (const char *formatstr)
 		{"links",	MU_CONFIG_FORMAT_LINKS},
 		{"plain",	MU_CONFIG_FORMAT_PLAIN},
 		{"sexp",	MU_CONFIG_FORMAT_SEXP},
+		{"json",	MU_CONFIG_FORMAT_JSON},
 		{"xml",		MU_CONFIG_FORMAT_XML},
 		{"xquery",	MU_CONFIG_FORMAT_XQUERY},
-		{"mquery",	MU_CONFIG_FORMAT_MQUERY}
+		{"mquery",	MU_CONFIG_FORMAT_MQUERY},
+		{"debug",	MU_CONFIG_FORMAT_DEBUG}
 	};
 
 	for (i = 0; i != G_N_ELEMENTS(formats); i++)
@@ -64,23 +67,28 @@ get_output_format (const char *formatstr)
 }
 
 
+#define expand_dir(D)			                     	\
+	if ((D)) {						\
+		char *exp;					\
+		exp = mu_util_dir_expand((D));			\
+		if (exp) {					\
+			g_free((D));				\
+			(D) = exp;				\
+		}						\
+	}
+
+
 static void
 set_group_mu_defaults (void)
 {
-	gchar *exp;
+	/* If muhome is not set, we use the XDG Base Directory Specification
+	 * locations. */
+	if (MU_CONFIG.muhome)
+                expand_dir(MU_CONFIG.muhome);
 
-	if (!MU_CONFIG.muhome)
-		MU_CONFIG.muhome = mu_util_guess_mu_homedir();
-
-	exp = mu_util_dir_expand(MU_CONFIG.muhome);
-	if (exp) {
-		g_free(MU_CONFIG.muhome);
-		MU_CONFIG.muhome = exp;
-	}
-
-	/* check for the MU_NOCOLOR env var; but in any case don't
+	/* check for the MU_NOCOLOR or NO_COLOR env vars; but in any case don't
 	 * use colors unless we're writing to a tty */
-	if (g_getenv (MU_NOCOLOR) != NULL)
+	if (g_getenv (MU_NOCOLOR) != NULL || g_getenv ("NO_COLOR") != NULL)
 		MU_CONFIG.nocolor = TRUE;
 
 	if (!isatty(fileno(stdout)) || !isatty(fileno(stderr)))
@@ -118,61 +126,75 @@ config_options_group_mu (void)
 	return og;
 }
 
-
-#define expand_dir(D)			                     	\
-	if ((D)) {						\
-		char *exp;					\
-		exp = mu_util_dir_expand((D));			\
-		if (exp) {					\
-			g_free((D));				\
-			(D) = exp;				\
-		}						\
-	}
-
-
-
 static void
-set_group_index_defaults (void)
+set_group_init_defaults (void)
 {
 	if (!MU_CONFIG.maildir)
-		MU_CONFIG.maildir = mu_util_guess_maildir ();
+		MU_CONFIG.maildir = mu_util_guess_maildir();
 
 	expand_dir (MU_CONFIG.maildir);
 }
 
+static GOptionGroup*
+config_options_group_init (void)
+{
+	GOptionGroup *og;
+	GOptionEntry entries[] = {
+		{"maildir", 'm', 0, G_OPTION_ARG_FILENAME, &MU_CONFIG.maildir,
+		 "top of the maildir", "<maildir>"},
+		{"my-address", 0, 0, G_OPTION_ARG_STRING_ARRAY,
+		 &MU_CONFIG.my_addresses, "my e-mail address; can be used multiple times",
+		 "<address>"},
+                {NULL, 0, 0, 0, NULL, NULL, NULL}
+	};
+
+	og = g_option_group_new("init", "Options for the 'index' command",
+				"", NULL, NULL);
+	g_option_group_add_entries(og, entries);
+
+	return og;
+}
+
+static gboolean
+index_post_parse_func (GOptionContext *context, GOptionGroup *group,
+                       gpointer data, GError **error)
+{
+        if (!MU_CONFIG.maildir && !MU_CONFIG.my_addresses)
+                return TRUE;
+
+        g_printerr ("%sNOTE%s: as of mu 1.3.8, 'mu index' no longer uses the\n"
+                    "--maildir/-m or --my-address options.\n\n",
+                    color_maybe(MU_COLOR_RED), color_maybe(MU_COLOR_DEFAULT));
+        g_printerr ("Instead, these options should be passed to 'mu init'.\n");
+        g_printerr ("See the mu-init(1) or the mu4e reference manual,\n'Initializing the message store' for details.\n\n");
+
+        return TRUE;
+}
+
+
 static GOptionGroup*
 config_options_group_index (void)
 {
 	GOptionGroup *og;
 	GOptionEntry entries[] = {
-		{"maildir", 'm', 0, G_OPTION_ARG_FILENAME, &MU_CONFIG.maildir,
-		 "top of the maildir", "<maildir>"},
-		{"rebuild", 0, 0, G_OPTION_ARG_NONE, &MU_CONFIG.rebuild,
-		 "rebuild the database from scratch (false)", NULL},
+                /* only here so we can tell users they are deprecated */
+                {"maildir", 'm', G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_FILENAME,
+                 &MU_CONFIG.maildir, "top of the maildir", "<maildir>"},
+		{"my-address", 0, G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_STRING_ARRAY,
+		 &MU_CONFIG.my_addresses, "my e-mail address; can be used multiple times",
+		 "<address>"},
+
 		{"lazy-check", 0, 0, G_OPTION_ARG_NONE, &MU_CONFIG.lazycheck,
 		 "only check dir-timestamps (false)", NULL},
-		{"my-address", 0, 0, G_OPTION_ARG_STRING_ARRAY,
-		 &MU_CONFIG.my_addresses,
-		 "my e-mail address (regexp); can be used multiple times",
-		 "<address>"},
-		{"autoupgrade", 0, 0, G_OPTION_ARG_NONE,
-		 &MU_CONFIG.autoupgrade,
-		 "auto-upgrade the database with new mu versions (false)",
-		 NULL},
 		{"nocleanup", 0, 0, G_OPTION_ARG_NONE, &MU_CONFIG.nocleanup,
 		 "don't clean up the database after indexing (false)", NULL},
-		{"xbatchsize", 0, 0, G_OPTION_ARG_INT, &MU_CONFIG.xbatchsize,
-		 "set transaction batchsize for xapian commits (0)", NULL},
-		{"max-msg-size", 0, 0, G_OPTION_ARG_INT,
-		 &MU_CONFIG.max_msg_size,
-		 "set the maximum size for message files", "<size>"},
 		{NULL, 0, 0, 0, NULL, NULL, NULL}
 	};
 
-	og = g_option_group_new("index",
-				"Options for the 'index' command",
+	og = g_option_group_new("index", "Options for the 'index' command",
 				"", NULL, NULL);
 	g_option_group_add_entries(og, entries);
+        g_option_group_set_parse_hooks(og, NULL, (GOptionParseFunc)index_post_parse_func);
 
 	return og;
 }
@@ -268,7 +290,6 @@ config_options_group_mkdir (void)
 	return og;
 }
 
-
 static void
 set_group_cfind_defaults (void)
 {
@@ -278,7 +299,6 @@ set_group_cfind_defaults (void)
 		MU_CONFIG.format  = get_output_format (MU_CONFIG.formatstr);
 }
 
-
 static GOptionGroup *
 config_options_group_cfind (void)
 {
@@ -301,8 +321,6 @@ config_options_group_cfind (void)
 	return og;
 }
 
-
-
 static GOptionGroup *
 config_options_group_script (void)
 {
@@ -321,9 +339,6 @@ config_options_group_script (void)
 	return og;
 }
 
-
-
-
 static void
 set_group_view_defaults (void)
 {
@@ -352,7 +367,6 @@ crypto_option_entries (void)
 	return entries;
 }
 
-
 static GOptionGroup *
 config_options_group_view (void)
 {
@@ -378,8 +392,6 @@ config_options_group_view (void)
 	return og;
 }
 
-
-
 static void
 set_group_extract_defaults (void)
 {
@@ -390,7 +402,6 @@ set_group_extract_defaults (void)
 }
 
 
-
 static GOptionGroup*
 config_options_group_extract (void)
 {
@@ -440,8 +451,10 @@ config_options_group_server (void)
 {
 	GOptionGroup *og;
 	GOptionEntry entries[] = {
-		{"maildir", 'm', 0, G_OPTION_ARG_FILENAME, &MU_CONFIG.maildir,
-		 "top of the maildir", "<maildir>"},
+                {"commands", 0, 0, G_OPTION_ARG_NONE, &MU_CONFIG.commands,
+		 "list the available command and their parameters, then exit", NULL},
+                {"eval", 'e', G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_STRING,
+                 &MU_CONFIG.eval, "expression to evaluate", "<expr>"},
 		{NULL, 0, 0, 0, NULL, NULL, NULL}
 	};
 
@@ -468,6 +481,8 @@ cmd_from_string (const char *str)
 		{ "find",    MU_CONFIG_CMD_FIND    },
 		{ "help",    MU_CONFIG_CMD_HELP    },
 		{ "index",   MU_CONFIG_CMD_INDEX   },
+                { "info",    MU_CONFIG_CMD_INFO    },
+                { "init",    MU_CONFIG_CMD_INIT    },
 		{ "mkdir",   MU_CONFIG_CMD_MKDIR   },
 		{ "remove",  MU_CONFIG_CMD_REMOVE  },
 		{ "script",  MU_CONFIG_CMD_SCRIPT  },
@@ -544,6 +559,8 @@ get_option_group (MuConfigCmd cmd)
 		return config_options_group_find();
 	case MU_CONFIG_CMD_INDEX:
 		return config_options_group_index();
+        case MU_CONFIG_CMD_INIT:
+		return config_options_group_init();
 	case MU_CONFIG_CMD_MKDIR:
 		return config_options_group_mkdir();
 	case MU_CONFIG_CMD_SERVER:
@@ -562,7 +579,7 @@ get_option_group (MuConfigCmd cmd)
 
 
 /* ugh yuck massaging the GOption text output; glib prepares some text
- * which has a 'Usage:' for the 'help' commmand. However, we need the
+ * which has a 'Usage:' for the 'help' command. However, we need the
  * help for the command we're asking help for. So, we remove the Usage:
  * from what glib generates. :-( */
 static gchar*
@@ -608,7 +625,7 @@ mu_config_show_help (MuConfigCmd cmd)
 {
 	GOptionContext *ctx;
 	GOptionGroup *group;
-	char *cleanhelp;
+	char *help, *cleanhelp;
 
 	g_return_if_fail (mu_config_cmd_is_valid(cmd));
 
@@ -619,15 +636,16 @@ mu_config_show_help (MuConfigCmd cmd)
 	if (group)
 		g_option_context_add_group (ctx, group);
 
-	g_option_context_set_description (ctx,
-					  get_help_string (cmd, TRUE));
-	cleanhelp = massage_help
-		(g_option_context_get_help (ctx, TRUE, group));
+	g_option_context_set_description (ctx, get_help_string (cmd, TRUE));
+	help	  = g_option_context_get_help (ctx, TRUE, group);
+	cleanhelp = massage_help (help);
 
 	g_print ("usage:\n\t%s%s",
 		 get_help_string (cmd, FALSE), cleanhelp);
 
+	g_free (help);
 	g_free (cleanhelp);
+	g_option_context_free (ctx);
 }
 
 static gboolean
@@ -717,7 +735,7 @@ mu_config_init (int *argcp, char ***argvp, GError **err)
 
 	/* fill in the defaults if user did not specify */
 	set_group_mu_defaults();
-	set_group_index_defaults();
+	set_group_init_defaults();
 	set_group_find_defaults();
 	set_group_cfind_defaults();
 	set_group_view_defaults();
@@ -750,13 +768,13 @@ mu_config_uninit (MuConfig *opts)
 	g_free (opts->targetdir);
 	g_free (opts->parts);
 	g_free (opts->script);
+        g_free (opts->eval);
 
 	g_strfreev (opts->params);
 
 	memset (opts, 0, sizeof(MU_CONFIG));
 }
 
-
 size_t
 mu_config_param_num (MuConfig *opts)
 {
