@@ -1,7 +1,5 @@
-/* -*-mode: c; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-*/
-
 /*
-** Copyright (C) 2008-2017 Dirk-Jan C. Binnema <djcb@djcbsoftware.nl>
+** Copyright (C) 2008-2020 Dirk-Jan C. Binnema <djcb@djcbsoftware.nl>
 **
 ** This program is free software; you can redistribute it and/or modify it
 ** under the terms of the GNU General Public License as published by the
@@ -26,7 +24,7 @@
 #include <sys/types.h> /* for mode_t */
 #include <mu-msg-fields.h>
 #include <mu-msg.h>
-#include <mu-util.h>
+#include <utils/mu-util.h>
 
 G_BEGIN_DECLS
 
@@ -34,7 +32,7 @@ G_BEGIN_DECLS
 #define MU_NOCOLOR "MU_NOCOLOR"
 
 
-enum _MuConfigFormat {
+typedef enum {
 	MU_CONFIG_FORMAT_UNKNOWN = 0,
 
 	/* for cfind, find, view */
@@ -47,9 +45,11 @@ enum _MuConfigFormat {
 	MU_CONFIG_FORMAT_CSV,		/* comma-sep'd values */
 	MU_CONFIG_FORMAT_ORG_CONTACT,	/* org-contact */
 	MU_CONFIG_FORMAT_BBDB,		/* BBDB */
+	MU_CONFIG_FORMAT_DEBUG,
 
 	/* for find, view */
 	MU_CONFIG_FORMAT_SEXP,		/* output sexps (emacs) */
+	MU_CONFIG_FORMAT_JSON,		/* output JSON */
 
 	/* for find */
 	MU_CONFIG_FORMAT_LINKS,		/* output as symlinks */
@@ -58,11 +58,9 @@ enum _MuConfigFormat {
 	MU_CONFIG_FORMAT_MQUERY,	/* output the mux query */
 
 	MU_CONFIG_FORMAT_EXEC		/* execute some command */
-};
-typedef enum _MuConfigFormat MuConfigFormat;
+} MuConfigFormat;
 
-
-enum _MuConfigCmd {
+typedef enum {
 	MU_CONFIG_CMD_UNKNOWN = 0,
 
 	MU_CONFIG_CMD_ADD,
@@ -71,6 +69,8 @@ enum _MuConfigCmd {
 	MU_CONFIG_CMD_FIND,
 	MU_CONFIG_CMD_HELP,
 	MU_CONFIG_CMD_INDEX,
+        MU_CONFIG_CMD_INFO,
+        MU_CONFIG_CMD_INIT,
 	MU_CONFIG_CMD_MKDIR,
 	MU_CONFIG_CMD_REMOVE,
 	MU_CONFIG_CMD_SCRIPT,
@@ -80,8 +80,7 @@ enum _MuConfigCmd {
 	MU_CONFIG_CMD_VIEW,
 
 	MU_CONFIG_CMD_NONE
-};
-typedef enum _MuConfigCmd MuConfigCmd;
+} MuConfigCmd;
 
 
 #define mu_config_cmd_is_valid(C)					\
@@ -108,21 +107,18 @@ struct _MuConfig {
 					 * in some output */
 	gboolean	 verbose;	/* verbose output */
 
+        /* options for init */
+        gchar	        *maildir;	/* where the mails are */
+        char**          my_addresses;   /* 'my e-mail address', for mu
+                                         * cfind; can be use multiple
+                                         * times */
 	/* options for indexing */
-	gchar	        *maildir;	/* where the mails are */
+
 	gboolean        nocleanup;	/* don't cleanup del'd mails from db */
 	gboolean        rebuild;	/* empty the database before indexing */
-	gboolean        autoupgrade;    /* automatically upgrade db
-					 * when needed */
 	gboolean        lazycheck;      /* don't check dirs with up-to-date
 					 * timestamps */
-	int             xbatchsize;     /* batchsize for xapian
-					 * commits, or 0 for
-					 * default */
 	int		max_msg_size;   /* maximum size for message files */
-	char**          my_addresses;   /* 'my e-mail address', for mu
-					 * cfind; can be use multiple
-					 * times */
 
 	/* options for querying 'find' (and view-> 'summary') */
 	gchar		*fields;	/* fields to show in output */
@@ -134,7 +130,7 @@ struct _MuConfig {
 	gboolean	 summary;	/* OBSOLETE: use summary_len */
 	int	         summary_len;   /* max # of lines for summary */
 
-	gchar            *bookmark;	/* use bookmark */
+	gchar           *bookmark;	/* use bookmark */
 	gchar		*formatstr;     /* output type for find
 					 * (plain,links,xml,json,sexp)
 					 * and view (plain, sexp) and cfind
@@ -151,7 +147,7 @@ struct _MuConfig {
 					  * in results */
 	/* for find and cind */
 	time_t            after;          /* only show messages or
-					   * adresses last seen after
+					   * addresses last seen after
 					   * T */
 	/* options for crypto
 	 * ie, 'view', 'extract' */
@@ -185,7 +181,12 @@ struct _MuConfig {
 	gboolean	 overwrite;	/* should we overwrite same-named files */
 	gboolean         play;          /* after saving, try to 'play'
 					 * (open) the attmnt using xdgopen */
-	/* options for mu-script */
+	/* for server */
+	gboolean        commands;        /* dump documentations for server
+					  * commands */
+        gchar          *eval;            /* command to evaluate */
+
+        /* options for mu-script */
 	gchar           *script;         /* script to run */
 	const char      **script_params; /* parameters for scripts */
 };
