@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2010-2013 Dirk-Jan C. Binnema  <djcb@djcbsoftware.nl>
+** Copyright (C) 2010-2017 Dirk-Jan C. Binnema  <djcb@djcbsoftware.nl>
 **
 ** This program is free software; you can redistribute it and/or modify it
 ** under the terms of the GNU General Public License as published by the
@@ -25,8 +25,8 @@
 #include <gdk/gdkkeysyms.h>
 #include <string.h>		/* for memset */
 
-#include <mu-util.h>
-#include <mu-store.h>
+#include <utils/mu-util.h>
+#include <mu-store.hh>
 #include <mu-runtime.h>
 #include <mu-index.h>
 
@@ -110,7 +110,7 @@ tool_button (const char *name)
 
 	icon = gtk_image_new_from_icon_name
 		(name, GTK_ICON_SIZE_SMALL_TOOLBAR);
-	
+
 	return gtk_menu_tool_button_new (icon, NULL);
 }
 
@@ -247,7 +247,7 @@ on_list_view_error (MugMsgListView * mlist, MugError err, MugData * mugdata)
 		msg = "Error in query";
 		break;
 	default:
-		msg = "Some error occured";
+		msg = "Some error occurred";
 		break;
 	}
 
@@ -331,8 +331,8 @@ mug_main_area (MugData * mugdata)
 	return mainarea;
 }
 
-GtkWidget *
-mug_shell (MugData * mugdata)
+static GtkWidget*
+mug_shell (MugData *mugdata)
 {
 	GtkWidget *vbox;
 
